<?php
trait WilokeGridInstagram{
	protected $cacheInstagram = 'wiloke_grid_instagram_';

	protected function renderInstagramItems($aItems, $item_style, $atts){
		$index = 0;
		$template = $this->template('template/', $item_style, 'instagram');

		$animation = (isset($atts['wil_animation']) && $atts['wil_animation'] != 'none') ? 'wow ' . $atts['wil_animation'] : '';
		$animationHover = (isset($atts['wil_animation_hover']) && $atts['wil_animation_hover'] != '' ) ? $atts['wil_animation_hover'] : '';
		
		$linkKey = $atts['shortcode']['click_action'] == 'wiloke_grid_magnific_popup' ?  'full' : 'link';
		ob_start();
		foreach ($aItems as $aItem) {
			$delay = $index * 0.15;
			?>
			<div class="grid-item <?php echo esc_attr($animation); ?>" data-wow-delay="<?php echo esc_attr($delay) ?>s">
				<div class="grid-item__inner">
					<div class="grid-item__content-wrapper">
						<?php include $template; ?>
					</div>
				</div>
			</div>
			<?php
		}
		$output = ob_get_contents();
		ob_end_clean();
		$output = apply_filters('wiloke_isotope_instagram_item', $output, $aItems, $atts);
		return $output;
	}

	public function fetchInstagramFeeds($url, $count)
	{
		$args = array( 'decompress' => false, 'timeout' => 30, 'sslverify'   => true );
		$rawdata = wp_remote_get( esc_url_raw($url), $args);

		if ( !is_wp_error($rawdata) )
		{
			$instagramImages = wp_remote_retrieve_body($rawdata);

			$oInstagrams = json_decode($instagramImages);

			$aFeeds = array();
			if ( $oInstagrams->meta->code === 200 )
			{
				$nextPageUrl = isset($oInstagrams->pagination) && isset($oInstagrams->pagination->next_url) ? $oInstagrams->pagination->next_url : '';

				for ( $i=0; $i<$count; $i++ )
				{
					if ( isset($oInstagrams->data[$i]) ) {

						if ( !empty($oInstagrams->data[$i]->caption) ){
							$title = $oInstagrams->data[$i]->caption->text;
						}elseif ( !empty($oInstagrams->data[$i]->location) ){
							$title = !empty($oInstagrams->data[$i]->location->name);
						}else{
							$title = $oInstagrams->data[$i]->user->username;
						}

						$aFeeds[] = array(
							'username'  => $oInstagrams->data[$i]->user->username,
							'thumbnail' => $oInstagrams->data[$i]->images->thumbnail->url,
							'title'     => $title,
							'link'      => $oInstagrams->data[$i]->link,
							'full'      => $oInstagrams->data[$i]->images->standard_resolution->url
						);
					}
				}
				return array(
					'content' 		=> $aFeeds,
					'nextPageUrl' 	=> $nextPageUrl
				);
			}
		}

		return false;
	}

	public function getInstagramUserID($info, $accessToken)
	{
		$args = array( 'decompress' => false, 'timeout' => 30, 'sslverify'   => true );
		$url = 'https://api.instagram.com/v1/users/search?q='.$info.'&access_token='.$accessToken;
		$oSearchProfile = wp_remote_get( esc_url_raw( $url ), $args);
		if ( !empty($oSearchProfile) && !is_wp_error($oSearchProfile) )
		{
			$oSearchProfile = wp_remote_retrieve_body($oSearchProfile);
			$oSearchProfile = json_decode($oSearchProfile);
			if ( $oSearchProfile->meta->code === 200 )
			{
				foreach ( $oSearchProfile->data as $oInfo )
				{
					if ( $oInfo->username === $info )
					{
						return $oInfo->id;
					}
				}
			}
		}
		return false;
	}

	public function getInstagramFeeds($info, $accessToken, $count=6, $type='self')
	{
		if ( $type == 'self' )
		{
			$url  = 'https://api.instagram.com/v1/users/'.$info.'/media/recent?access_token='.$accessToken.'&count='.$count;
			return $this->fetchInstagramFeeds($url, $count);
		}else{
			$userID = $this->getInstagramUserID($info, $accessToken);
			if ( !empty($userID) )
			{
				$url  = 'https://api.instagram.com/v1/users/'.$userID.'/media/recent?access_token='.$accessToken.'&count='.$count;
				return $this->fetchInstagramFeeds($url, $count);
			}
		}
	}

	public function setInstagramCache($userName, $aInstagramSettings, $aData){
		if ( empty($aInstagramSettings['cache_interval']) ){
			return false;
		}

		$aCache = get_option($this->cacheInstagram.$userName);
		if ( empty($aCache) ){
			$aNewCache = $aData;
		}else{
			$aNewCache = array_merge($aCache, $aData);
		}

		update_option($this->cacheInstagram.$userName, maybe_serialize($aNewCache));
	}

	public function getInstagramCache($userName, $numberOfItems, $offset=0){
		if ( empty($aInstagramSettings['cache_interval']) ){
			return false;
		}

		$aCache = get_option($this->cacheInstagram.$userName);
		if ( empty($aCache) ){
			return false;
		}

		return array_slice($aCache, $offset, $numberOfItems);
	}

	public function instagramOutput($data){
		$aInstagramSettings = get_option('_pi_instagram_settings');
		if ( !empty($data['instagramUserName']) )
		{
			$type = 'username';
			$info = $data['instagramUserName'];
		}else{
			$type = 'self';
			$info = $aInstagramSettings['userid'];
		}

		if ( isset($data['nextInstagramPageUrl']) && !empty($data['nextInstagramPageUrl']) ){
			$aFeeds = $this->getInstagramCache($info, $data['posts_per_page'], $data['paged']);
			$nextPageUrl = '';
			if ( empty($aFeeds) ){
				$aInstagramData = $this->fetchInstagramFeeds($data['nextInstagramPageUrl'], $data['posts_per_page']);

				if ( !empty($aInstagramData) ){
					$this->setInstagramCache($info, $aInstagramSettings, $aInstagramData['content']);
				}

				$aFeeds = $aInstagramData['content'];
				$nextPageUrl = $aInstagramData['nextPageUrl'];
			}

			if ( !empty($aFeeds) ){
				$html = $this->renderInstagramItems($aFeeds, $data['item'], $data);
				$loadedAll = empty($nextPageUrl) ? true : false;
				$return = array(
					'status' 		=> 200,
					'data'	 		=> $html,
					'paged'			=> $data['paged'],
					'nextPageUrl'	=> $nextPageUrl,
					'loadedAll'		=> $loadedAll,
					'message'		=> esc_html__('Susscess!', 'wiloke-grid')
				);
			}else{
				$return['message'] = esc_html__('All images are loaded', 'wiloke-grid');
			}
		}else{
			$cacheInstagram = null;
			if ( empty($aInstagramSettings['access_token']) )
			{
				$return['message'] = esc_html__('Please go to Wiloke Grid => Providing your Instagram API', 'wiloke-grid');
			}else{
				$nextPageUrl = '';
				$aFeeds = $this->getInstagramCache($info, $data['posts_per_page']);
				if ( empty($aFeeds) ){
					$aInstagramData = $this->getInstagramFeeds($info, $aInstagramSettings['access_token'], $data['posts_per_page'], $type);
					if ( isset($aInstagramData['content']) ){
						$aFeeds = $aInstagramData['content'];
					}
					if ( !empty($aInstagramData) ){
						$this->setInstagramCache($info, $aInstagramSettings, $aInstagramData['content']);
						$nextPageUrl = $aInstagramData['nextPageUrl'];
						$aFeeds = $aInstagramData['content'];
					}
				}

				if ( !empty($aFeeds) ){
					$html = $this->renderInstagramItems($aFeeds, $data['item'], $data);
					$loadedAll = empty($nextPageUrl) ? true : false;
					$return = array(
						'status' 		=> 200,
						'data'	 		=> $html,
						'paged'			=> 1,
						'nextPageUrl'	=> $nextPageUrl,
						'loadedAll'		=> $loadedAll,
						'message'		=> esc_html__('Susscess!', 'wiloke-grid')
					);
				}else{
					$return['message'] = esc_html__('There are no images!', 'wiloke-grid');
				}
			}
		}

		return $return;
	}
}