<?php

trait WilokeGridYoutube {
	public function renderYoutubeItem($aItem, $itemStyle, $aShortcodeSettings){
		$this->template('template/', $itemStyle, 'youtube');
	}

	public function youtubeOutput($data){
		$aSettings = get_option('_pi_google_settings');

		if ( empty($aSettings) ){
			return array(
				'message' => esc_html__('Google API key is required: Wiloke Grid -> Google API Settings', 'wiloke-grid')
			);
		}

		$client = new Google_Client();
		$client->setDeveloperKey($aSettings['api']);
		$oYoutube = new Google_Service_YouTube($client);
		$nextPageToken = isset($data['youtubeNextpageToken']) ? $data['youtubeNextpageToken'] : '';

		try{
			$aPlaylistItemsResponse = $oYoutube->playlistItems->listPlaylistItems('snippet', array(
				'playlistId' => $data['shortcode']['wil_youtube_playlist_id'],
				'maxResults' => $data['posts_per_page'],
				'pageToken' => $nextPageToken));
			if ( empty($aPlaylistItemsResponse) ){
				return array(
					'message' => esc_html__('There are no videos in this playlist', 'wiloke-grid')
				);
			}

			$animation = (isset($data['shortcode']['wil_animation']) && $data['shortcode']['wil_animation'] != 'none') ? 'wow ' . $data['shortcode']['wil_animation'] : '';
			$animationHover = (isset($data['shortcode']['wil_animation_hover']) && $data['shortcode']['wil_animation_hover'] != '' ) ? $data['shortcode']['wil_animation_hover'] : '';
			$index = 0;
			$template = $this->template('template/', $data['shortcode']['item_style'], 'youtube');
			ob_start();
			foreach ($aPlaylistItemsResponse['items'] as $aItem) {
				$delay = $index * 0.15;
				?>
				<div class="grid-item <?php echo esc_attr($animation) ?>" data-wow-delay="<?php echo esc_attr($delay) ?>s">
					<div class="grid-item__inner">
						<div class="grid-item__content-wrapper">
							<?php include $template; ?>
						</div>
					</div>
				</div>
				<?php
				$index++;
			}
			$output = ob_get_contents();
			ob_end_clean();
			$nextPageToken = $aPlaylistItemsResponse['nextPageToken'];
			$output = apply_filters('wiloke_isotope_youtube_item', $output, $aPlaylistItemsResponse, $data);

			return array(
				'status' 		=> 200,
				'data'	 		=> $output,
				'paged'			=> $data['paged'],
				'nextPageToken'	=> $nextPageToken,
				'loadedAll'		=> empty($nextPageToken),
				'message'		=> esc_html__('Success!', 'wiloke-grid')
			);
		}catch (Exception $oException){
			return array(
				'message' => esc_html__('Wrong Listing ID', 'wiloke-grid')
			);
		}
	}
}