<?php
if( !class_exists('WilokeGridAdmin') ) {

	class WilokeGridAdmin {
		
		public function __construct() {

			add_action( 'admin_init', array($this, 'shortcodeParam'));

			vc_add_shortcode_param( 'wiloke_grid', array($this, 'gridField'));
			vc_add_shortcode_param( 'wiloke_checkbox', array($this, 'checkboxsField' ));

			// Autocomplete
			add_filter('vc_autocomplete_wil_shortcode_grid_include_callback', array($this, 'vc_include_field_search'), 10, 3);
			add_filter('vc_autocomplete_wil_shortcode_grid_include_render', array($this, 'vc_include_field_render'), 10, 1);

			add_action('vc_backend_editor_enqueue_js_css', array($this, 'enqueue_scripts_admin') );
			add_action('vc_frontend_editor_enqueue_js_css', array($this, 'enqueue_scripts_admin') );

			add_action( 'wp_ajax_wil_grid_taxonomies', array( $this, 'getAjaxTaxonomies') );
			// add_action( 'wp_ajax_nopriv_wil_grid_taxonomies', array( $this, 'getAjaxTaxonomies') );

			add_action( 'wp_ajax_wil_grid_terms', array( $this, 'getAjaxTerms') );
			// add_action( 'wp_ajax_nopriv_wil_grid_terms', array( $this, 'getAjaxTerms') );

			add_action( 'wp_ajax_wil_grid_save_tmpl', array( $this, 'saveAjaxTemplate') );
			// add_action( 'wp_ajax_nopriv_wil_grid_save_tmpl', array( $this, 'saveAjaxTemplate') );

			add_action( 'wp_ajax_wil_grid_get_tmpl', array( $this, 'getAjaxTemplate') );
			// add_action( 'wp_ajax_nopriv_wil_grid_get_tmpl', array( $this, 'getAjaxTemplate') );

			add_action( 'wp_ajax_wil_grid_remove_tmpl', array( $this, 'removeAjaxTemplate') );
			// add_action( 'wp_ajax_nopriv_wil_grid_remove_tmpl', array( $this, 'removeAjaxTemplate') );

			add_action('admin_footer', array($this, 'template'));

			add_action('admin_menu', array($this, 'registerMenu'));
			add_action('admin_enqueue_scripts', array($this, 'adminScripts'));
		}

		public function adminScripts($hook){
			if ( $hook !== 'toplevel_page_wiloke-grid' ){
				return false;
			}

			wp_enqueue_style('semantic-ui', plugin_dir_url(__FILE__) . 'assets/css/semantic.min.css');
			wp_enqueue_script('semantic-ui', plugin_dir_url(__FILE__) . 'assets/js/lib/semantic.min.js', array('jquery'), null, true);
		}

		public function registerMenu(){
			add_menu_page('Wiloke Grid', 'Wiloke Grid', 'edit_theme_options', 'wiloke-grid', array($this, 'wilokeGridSettings'));
		}

		public function itemHoverEffect(){
			$aEffects = array();
			for ( $i = 0; $i <=27; $i++ ){
				$aEffects['effect-'.$i] = 'effect-'.$i;
			}
			return $aEffects;
		}

        public function updateSettings($key, $aData)
        {
            $aOldVal = get_option($key);
            $aData = !empty($aOldVal) ? array_merge($aOldVal, $aData) : $aData;

            $aNewData = array();
            foreach ($aData as $order => $value) {
            	$aNewData[sanitize_text_field($order)] = sanitize_text_field($value);
            }

            update_option($key, $aNewData);
        }

        public function wilokeGridSettings()
        {
            if ( !current_user_can('edit_theme_options') ){
                return false;
            }

            ?>
            <div style="max-width: 1200px; margin: 20px auto;">
	            <form class="form ui" method="POST" action="<?php echo admin_url('admin.php?page=wiloke-grid'); ?>">
	            	<?php 
	            	include plugin_dir_path(__FILE__) . 'settings/instagram.php';
		            //include plugin_dir_path(__FILE__) . 'settings/facebook.php';
		            include plugin_dir_path(__FILE__) . 'settings/google.php';
		            include plugin_dir_path(__FILE__) . 'settings/vimeo.php';
	            	?>
	            </form>
            </div>
            <?php
        }

		public function vc_include_field_search($search_string, $tag, $aParam){
			$query = $search_string;
			$data = array();
			$args = array(
				's' => $query,
				'post_type' => 'any',
			);
			$args['vc_search_by_title_only'] = true;
			$args['numberposts'] = - 1;
			if ( 0 === strlen( $args['s'] ) ) {
				unset( $args['s'] );
			}

			$posts = get_posts( $args );
			if ( is_array( $posts ) && ! empty( $posts ) ) {
				foreach ( $posts as $post ) {
					$data[] = array(
						'value' => $post->ID,
						'label' => $post->post_title,
						'group' => $post->post_type,
					);
				}
			}

			return $data;
		}

		/**
		 * @param $value
		 *
		 * @return array|bool
		 */
		public function vc_include_field_render( $value ) {
			$post = get_post( $value['value'] );

			return is_null( $post ) ? false : array(
				'label' => $post->post_title,
				'value' => $post->ID,
				'group' => $post->post_type,
			);
		}

		// vc Script
		public function enqueue_scripts_admin() {
		    wp_enqueue_style( 'wiloke_grid', WGRID_URI . 'admin/assets/css/wiloke.grid.css', false, '1.0.0' );
		    wp_enqueue_script( 'draggabilly', WGRID_URI . 'admin/assets/js/lib/draggabilly.pkgd.min.js', array(), '1.0', true );
		    wp_enqueue_script( 'packery', WGRID_URI . 'admin/assets/js/lib/packery.pkgd.min.js', array(), '1.0', true );
		    wp_enqueue_script( 'wiloke.resizable', WGRID_URI . 'admin/assets/js/lib/wiloke.resizable.js', array(), '1.0', true );
		    wp_enqueue_script( 'wiloke.packery', WGRID_URI . 'admin/assets/js/lib/wiloke.packery.js', array(), '1.0', true );
		    wp_enqueue_script( 'wiloke_grid_js', WGRID_URI . 'admin/assets/js/lib/wiloke.grid.js', array(), '1.0', true );
		    wp_enqueue_script( 'wiloke_grid_script', WGRID_URI . 'admin/assets/js/wiloke.grid.script.js', array(), '1.0', true );
		}

		public function getAjaxTaxonomies($post_type) {
			if ( isset($_GET['post_type']) ) {
				$return = array();
				$taxonomies = get_object_taxonomies( $_GET['post_type'], 'objects' );
				foreach ($taxonomies as $key => $taxonomy) {
					if ( 'post_format' !== $key && is_object( $taxonomy ) ) {
						$return[$key]= ucfirst($taxonomy->labels->name . '('. $key .')');
					}
				}
				wp_send_json($return);
			}
		}

		public function getAjaxTerms($taxonomy) {
			if ( isset($_GET['taxonomy']) ) {
				$return = array();
				$terms = get_terms( $_GET['taxonomy'], array(
					'hide_empty' => 0
				));
				if ( ! empty( $terms ) && ! is_wp_error( $terms ) ) {
				 	foreach ( $terms as $term ) {
					 	$return[$term->term_id]= ucfirst($term->name);
				 	}
				}
				wp_send_json($return);
			}
		}

		public function getAjaxTemplate() {
			$return  = array(
				'status'	=> 204,
				'message'	=> esc_html__('Error!', 'wiloke-grid')
			);
			if( check_ajax_referer( 'wiloke_grid_nonce_tmpl', 'security', false) ) {
				if( isset($_POST['key']) ) {
					$key = $_POST['key'];
					$data = get_option('wiloke_grid_template', false);
					if( isset($data[$key]) ) {
						$return  = array(
							'status'	=> 200,
							'models'		=> $data[$key]['models'],
							'message'	=> esc_html__('Susscess!', 'wiloke-grid')
						);
					} else {
						$return['message'] = esc_html__('Empty!', 'wiloke-grid');
					}
				}
			}
			wp_send_json($return);
		}

		public function saveAjaxTemplate() {

			$return  = array(
				'status'	=> 204,
				'message'	=> esc_html__('Error!', 'wiloke-grid')
			);

			if( check_ajax_referer( 'wiloke_grid_nonce_create_tmpl', 'security', false) ) {
				if( isset($_POST['data']) ) {
					$data = $_POST['data'];
					$option = get_option('wiloke_grid_template', false);
					if( $option ) {
						$option[$data['id']] = $data;
					} else {

						$option = array(
							$data['id'] => $data
						);
					}
					update_option('wiloke_grid_template', $option);
					$return = array(
						'status'	=> 200,
						'message'	=> esc_html__('Susscess!', 'wiloke-grid')
					);
				}
			}

			wp_send_json($return);
		}

		public function removeAjaxTemplate() {

			$return  = array(
				'status'	=> 204,
				'message'	=> esc_html__('Error!', 'wiloke-grid')
			);

			if( check_ajax_referer( 'wiloke_grid_nonce_tmpl', 'security', false) ) {

				if( isset($_POST['key']) ) {

					$key = $_POST['key'];
					$data = get_option('wiloke_grid_template', false);

					if( isset($data[$key]) ) {

						unset($data[$key]);

						update_option('wiloke_grid_template', $data);

						$return  = array(
							'status'	=> 200,
							'message'	=> esc_html__('Susscess!', 'wiloke-grid')
						);

					} else {
						$return['message'] = esc_html__('Empty!', 'wiloke-grid');
					}
				}
			}

			wp_send_json($return);
		}

		protected function postTypes() {

			$postTypesList = array();

			$postTypes = get_post_types( array(
				'public' => true,
			));

			if ( is_array( $postTypes ) && ! empty( $postTypes ) ) {

				foreach ( $postTypes as $postType ) {

					if ( !in_array( $postType, array('attachment', 'page') ) ) {

						$label = ucfirst( $postType );

						$postTypesList[] = array(
							$postType,
							$label,
						);
					}
				}
			}

			$postTypesList['Specify Post IDs'] = 'ids';
			$postTypesList['Instagram'] = 'instagram';
			// $postTypesList['Facebook'] = 'facebook';
			$postTypesList['Youtube'] = 'youtube';
			$postTypesList['Vimeo'] = 'vimeo';

			return $postTypesList;
		}

		protected function itemStyle() {
			
			$array = array(
				esc_html__('Wiloke Default', 'wiloke-grid')		=> 'default'
			);

			$array = apply_filters('wiloke_grid_add_item', $array);

			return $array;
		}

		protected function itemAnimation() {

			return array(
				esc_html__( 'None', 'wiloke-grid' ) 					=> 'none',
				esc_html__( 'Bounce', 'wiloke-grid' ) 				=> 'bounce',
				esc_html__( 'Flash', 'wiloke-grid' ) 					=> 'flash',
				esc_html__( 'Pulse', 'wiloke-grid' ) 					=> 'pulse',
				esc_html__( 'RubberBand', 'wiloke-grid' ) 			=> 'rubberBand',
				esc_html__( 'Shake', 'wiloke-grid' ) 					=> 'shake',
				esc_html__( 'Swing', 'wiloke-grid' ) 					=> 'swing',
				esc_html__( 'Tada', 'wiloke-grid' ) 					=> 'tada',
				esc_html__( 'Wobble', 'wiloke-grid' ) 				=> 'wobble',
				esc_html__( 'Jello', 'wiloke-grid' ) 					=> 'jello',
				esc_html__( 'Bounce In', 'wiloke-grid' ) 				=> 'bounceIn',
				esc_html__( 'Bounce InDown', 'wiloke-grid' ) 			=> 'bounceInDown',
				esc_html__( 'Bounce InLeft', 'wiloke-grid' ) 			=> 'bounceInLeft',
				esc_html__( 'Bounce InRight', 'wiloke-grid' ) 		=> 'bounceInRight',
				esc_html__( 'Bounce InUp', 'wiloke-grid' ) 			=> 'bounceInUp',
				esc_html__( 'Bounce Out', 'wiloke-grid' ) 			=> 'bounceOut',
				esc_html__( 'Bounce OutDown', 'wiloke-grid' ) 		=> 'bounceOutDown',
				esc_html__( 'Bounce OutLeft', 'wiloke-grid' ) 		=> 'bounceOutLeft',
				esc_html__( 'Bounce OutRight', 'wiloke-grid' ) 		=> 'bounceOutRight',
				esc_html__( 'Bounce OutUp', 'wiloke-grid' ) 			=> 'bounceOutUp',
				esc_html__( 'Fade In', 'wiloke-grid' ) 				=> 'fadeIn',
				esc_html__( 'Fade InDown', 'wiloke-grid' ) 			=> 'fadeInDown',
				esc_html__( 'Fade InDown Big', 'wiloke-grid' ) 		=> 'fadeInDownBig',
				esc_html__( 'Fade InLeft', 'wiloke-grid' ) 			=> 'fadeInLeft',
				esc_html__( 'Fade InLeft Big', 'wiloke-grid' ) 		=> 'fadeInLeftBig',
				esc_html__( 'Fade InRight', 'wiloke-grid' ) 			=> 'fadeInRight',
				esc_html__( 'Fade InRight Big', 'wiloke-grid' ) 		=> 'fadeInRightBig',
				esc_html__( 'Fade InUp', 'wiloke-grid' ) 				=> 'fadeInUp',
				esc_html__( 'Fade InUp Big', 'wiloke-grid' ) 			=> 'fadeInUpBig',
				esc_html__( 'Fade Out', 'wiloke-grid' ) 				=> 'fadeOut',
				esc_html__( 'Fade OutDown', 'wiloke-grid' ) 			=> 'fadeOutDown',
				esc_html__( 'Fade OutDown Big', 'wiloke-grid' ) 		=> 'fadeOutDownBig',
				esc_html__( 'Fade OutLeft', 'wiloke-grid' ) 			=> 'fadeOutLeft',
				esc_html__( 'Fade OutLeft Big', 'wiloke-grid' ) 		=> 'fadeOutLeftBig',
				esc_html__( 'Fade OutRight', 'wiloke-grid' ) 			=> 'fadeOutRight',
				esc_html__( 'Fade OutRight Big', 'wiloke-grid' ) 		=> 'fadeOutRightBig',
				esc_html__( 'Fade OutUp', 'wiloke-grid' ) 			=> 'fadeOutUp',
				esc_html__( 'Fade OutUp Big', 'wiloke-grid' ) 		=> 'fadeOutUpBig',
				esc_html__( 'Flip', 'wiloke-grid' ) 					=> 'flip',
				esc_html__( 'Flip InX', 'wiloke-grid' ) 				=> 'flipInX',
				esc_html__( 'Flip InY', 'wiloke-grid' ) 				=> 'flipInY',
				esc_html__( 'Flip OutX', 'wiloke-grid' ) 				=> 'flipOutX',
				esc_html__( 'Flip OutY', 'wiloke-grid' ) 				=> 'flipOutY',
				esc_html__( 'Light Speed In', 'wiloke-grid' ) 		=> 'lightSpeedIn',
				esc_html__( 'Light Speed Out', 'wiloke-grid' ) 		=> 'lightSpeedOut',
				esc_html__( 'Rotate In', 'wiloke-grid' ) 				=> 'rotateIn',
				esc_html__( 'Rotate InDown Left', 'wiloke-grid' ) 	=> 'rotateInDownLeft',
				esc_html__( 'Rotate InDown Right', 'wiloke-grid' ) 	=> 'rotateInDownRight',
				esc_html__( 'Rotate InUp Left', 'wiloke-grid' ) 		=> 'rotateInUpLeft',
				esc_html__( 'Rotate InUp Right', 'wiloke-grid' ) 		=> 'rotateInUpRight',
				esc_html__( 'Rotate Out', 'wiloke-grid' ) 			=> 'rotateOut',
				esc_html__( 'Rotate OutDown Left', 'wiloke-grid' ) 	=> 'rotateOutDownLeft',
				esc_html__( 'Rotate OutDown Right', 'wiloke-grid' ) 	=> 'rotateOutDownRight',
				esc_html__( 'Rotate OutUp Left', 'wiloke-grid' ) 		=> 'rotateOutUpLeft',
				esc_html__( 'Rotate OutUp Right', 'wiloke-grid' ) 	=> 'rotateOutUpRight',
				esc_html__( 'Slide InUp', 'wiloke-grid' ) 			=> 'slideInUp',
				esc_html__( 'Slide InDown', 'wiloke-grid' ) 			=> 'slideInDown',
				esc_html__( 'Slide InLeft', 'wiloke-grid' ) 			=> 'slideInLeft',
				esc_html__( 'Slide InRight', 'wiloke-grid' ) 			=> 'slideInRight',
				esc_html__( 'Slide OutUp', 'wiloke-grid' ) 			=> 'slideOutUp',
				esc_html__( 'Slide OutDown', 'wiloke-grid' ) 			=> 'slideOutDown',
				esc_html__( 'Slide OutLeft', 'wiloke-grid' ) 			=> 'slideOutLeft',
				esc_html__( 'Slide OutRight', 'wiloke-grid' ) 		=> 'slideOutRight',
				esc_html__( 'Zoom In', 'wiloke-grid' ) 				=> 'zoomIn',
				esc_html__( 'Zoom InDown', 'wiloke-grid' ) 			=> 'zoomInDown',
				esc_html__( 'Zoom InLeft', 'wiloke-grid' ) 			=> 'zoomInLeft',
				esc_html__( 'Zoom InRight', 'wiloke-grid' ) 			=> 'zoomInRight',
				esc_html__( 'Zoom Out', 'wiloke-grid' ) 				=> 'zoomOut',
				esc_html__( 'Zoom OutDown', 'wiloke-grid' ) 			=> 'zoomOutDown',
				esc_html__( 'Zoom OutLeft', 'wiloke-grid' ) 			=> 'zoomOutLeft',
				esc_html__( 'Zoom OutRight', 'wiloke-grid' ) 			=> 'zoomOutRight',
				esc_html__( 'Zoom OutUp', 'wiloke-grid' ) 			=> 'zoomOutUp',
				esc_html__( 'Hinge', 'wiloke-grid' ) 					=> 'hinge',
				esc_html__( 'Roll In', 'wiloke-grid' ) 				=> 'rollIn',
				esc_html__( 'Roll Out', 'wiloke-grid' ) 				=> 'rollOut',
			);
		}

		protected function itemAnimationHover() {

			$animateHover =  array(
				esc_html__( 'None', 'wiloke-grid' ) 			=> '',
				esc_html__( 'Classic', 'wiloke-grid' ) 		=> 'classic',
				esc_html__( 'Fade', 'wiloke-grid' ) 			=> 'fade',
				esc_html__( 'FadeUp', 'wiloke-grid' ) 		=> 'fadeUp',
				esc_html__( 'FadeRight', 'wiloke-grid' ) 		=> 'fadeRight',
				esc_html__( 'FadeDown', 'wiloke-grid' ) 		=> 'fadeDown',
				esc_html__( 'FadeLeft', 'wiloke-grid' ) 		=> 'fadeLeft',
				esc_html__( 'FlipX', 'wiloke-grid' ) 			=> 'flipX',
				esc_html__( 'FlipY', 'wiloke-grid' ) 			=> 'flipY',
				esc_html__( 'PushRight', 'wiloke-grid' ) 		=> 'pushRight',
				esc_html__( 'PushTop', 'wiloke-grid' ) 		=> 'pushTop',
				esc_html__( 'PullBottom', 'wiloke-grid' ) 	=> 'pullBottom',
				esc_html__( 'PushLeft', 'wiloke-grid' ) 		=> 'pushLeft',
				esc_html__( 'SlideLeft', 'wiloke-grid' ) 		=> 'slideLeft',
				esc_html__( 'SlideRight', 'wiloke-grid' ) 	=> 'slideRight',
				esc_html__( 'SlideTop', 'wiloke-grid' ) 		=> 'slideTop',
				esc_html__( 'SlideBottom', 'wiloke-grid' ) 	=> 'slideBottom',
			);

			$animateHover = apply_filters('wiloke_animation_hover', $animateHover);

			return $animateHover;
		}

		protected function filtersStyle() {
			$array = array(
				esc_html__('Default', 'wiloke-grid')	=> 'wil_filter--default'
			);

			$array = apply_filters('wiloke_grid_filter_style', $array);

			return $array;
		}

		public function shortcodeParam() {

		    vc_map( array(
		        'name' => esc_html__( 'Wiloke Grid', 'wiloke-grid' ),
		        'base' => 'wil_shortcode_grid',
		        'category' => esc_html__('Wiloke', 'wiloke-grid' ),
		        'params' => array(
		            array(
						'type' => 'dropdown',
						'heading' => esc_html__( 'Data source', 'wiloke-grid' ),
						'param_name' => 'wil_post_type',
						'value' => $this->postTypes(),
						'save_always' => true,
						'description' => esc_html__( 'Select content type for your grid. If you want to use Instagram, please make your Instagram API has been provided: Wiloke Grid -> Instagram', 'wiloke-grid' ),
						'admin_label' => true,
					),
					array(
						'type' 			=> 'autocomplete',
						'heading' 		=> esc_html__( 'Enter in your post names', 'wiloke-grid' ),
						'param_name' 	=> 'include',
						'description' 	=> esc_html__( 'Add posts, pages, etc. by title.', 'wiloke-grid' ),
						'settings' 		=> array(
							'multiple' 	=> true,
							'sortable' 	=> true,
							'groups' 	=> true,
						),
						'dependency' => array(
							'element' 	=> 'wil_post_type',
							'value' 	=> array( 'ids' ),
						),
					),
					array(
						'type' => 'dropdown',
						'heading' => esc_html__( 'Taxonomies', 'wiloke-grid' ),
						'param_name' => 'wil_taxonomy',
						'value'	=> array(),
						'save_always' => true,
						'description' => esc_html__( 'Enter categories, tags or custom taxonomies.', 'wiloke-grid' ),
						'dependency' => array(
							'element' => 'wil_post_type',
							'value_not_equal_to' => array( 'ids', 'instagram', 'facebook', 'youtube', 'vimeo' ),
							'callback' => 'WilGridTaxonomiesCallBack'
						)
					),
			        array(
				        'type' 			=> 'textfield',
				        'heading' 		=> 'Vimeo Album ID',
				        'param_name' 	=> 'wil_vimeo_album_id',
				        'dependency' 	=> array(
					        'element'	=> 'wil_post_type',
					        'value' 	=> array('vimeo')
				        )
			        ),
					array(
						'type' 			=> 'textfield',
						'heading' 		=> 'Instagram Username',
						'param_name' 	=> 'wil_instagram_username',
						'description' 	=> 'Leave empty to use the default setting in Wiloke Grid',
						'dependency' 	=> array(
							'element'	=> 'wil_post_type',
							'value' 	=> array('instagram')
						)
					),
			        array(
				        'type' 			=> 'textfield',
				        'heading' 		=> 'Youtube Playlist ID',
				        'param_name' 	=> 'wil_youtube_playlist_id',
				        'dependency' 	=> array(
					        'element'	=> 'wil_post_type',
					        'value' 	=> array('youtube')
				        )
			        ),
			        array(
				        'type' 			=> 'dropdown',
				        'heading' 		=> esc_html__( 'Thumbnail Size', 'wiloke-grid' ),
				        'param_name' 	=> 'wil_thumbnail_size',
				        'dependency' 	=> array(
					        'element'	=> 'wil_post_type',
					        'value' 	=> array('youtube')
				        ),
                        'default'       => 'high',
                        'value' => array(
                            'Small'             => 'default',
                            'Medium'            => 'medium',
                            'High'              => 'high',
                            'Max Resolution'    => 'maxres',
                        )
			        ),
					array(
						'type' 			=> 'wiloke_checkbox',
						'heading' 		=> esc_html__( 'Exclude from taxonomies', 'wiloke-grid' ),
						'param_name' 	=> 'wil_terms',
						'description' 	=> esc_html__( 'Enter categories, tags won\'t be shown in the filters list', 'wiloke-grid' ),
						'dependency' 	=> array(
							'element'				=> 'wil_post_type',
							'value_not_equal_to' 	=> array( 'ids', 'instagram', 'facebook', 'youtube', 'vimeo' ),
							'callback' 				=> 'WilGridTaxonomiesExcludeCallBack'
						)
					),

					array(
						'type' => 'dropdown',
						'heading' => esc_html__( 'Show Filter', 'wiloke-grid' ),
						'param_name' => 'wil_show_filter',
						'value' => array(
							esc_html__('Yes', 'wiloke-grid')		=> 'yes',
							esc_html__('No', 'wiloke-grid')		=> 'no'
						),
						'dependency' => array(
							'element' => 'wil_post_type',
							'value_not_equal_to' => array( 'ids', 'instagram', 'facebook', 'youtube', 'vimeo' ),
						),
						'description' => esc_html__( 'Append filter to grid.', 'wiloke-grid' ),
					),

					array(
						'type' => 'dropdown',
						'heading' => esc_html__( 'Filter Text Color', 'wiloke-grid' ),
						'param_name' => 'wil_filter_text_color',
						'value' => array(
							'Default' 		=> 'wil-default',
							'Dark'			=> 'wil-dark',
							'Light'			=> 'wil-light',
							'Custom Color'	=> 'custom_color'
						),
						'dependency' => array(
							'element' => 'wil_show_filter',
							'value'   => array( 'yes' ),
						)
					),

					array(
						'type' => 'colorpicker',
						'heading' => esc_html__( 'Filter Text Color', 'wiloke-grid' ),
						'param_name' => 'wil_filter_custom_text_color',
						'dependency' => array(
							'element' => 'wil_filter_text_color',
							'value'   => array( 'custom_color' ),
						)
					),

					array(
						'type' => 'dropdown',
						'heading' => esc_html__( 'Order by', 'wiloke-grid' ),
						'param_name' => 'wil_orderby',
						'value' => array(
							esc_html__( 'Date', 'wiloke-grid' ) => 'date',
							esc_html__( 'Order by post ID', 'wiloke-grid' ) => 'ID',
							esc_html__( 'Author', 'wiloke-grid' ) => 'author',
							esc_html__( 'Title', 'wiloke-grid' ) => 'title',
							esc_html__( 'Last modified date', 'wiloke-grid' ) => 'modified',
							esc_html__( 'Number of comments', 'wiloke-grid' ) => 'comment_count',
							esc_html__( 'Random order', 'wiloke-grid' ) => 'rand',
						),
						'dependency' => array(
							'element' => 'wil_post_type',
							'value_not_equal_to' => array( 'ids', 'instagram', 'youtube', 'vimeo' ),
						),
						'description' => esc_html__( 'Select order type.', 'wiloke-grid' ),
					),

					array(
						'type' => 'dropdown',
						'heading' => esc_html__( 'Sort order', 'wiloke-grid' ),
						'param_name' => 'wil_order',
						'value' => array(
							esc_html__( 'Descending', 'wiloke-grid' ) => 'DESC',
							esc_html__( 'Ascending', 'wiloke-grid' ) => 'ASC',
						),
						'dependency' => array(
							'element' => 'wil_post_type',
							'value_not_equal_to' => array( 'instagram', 'facebook', 'youtube', 'vimeo' ),
						),
						'description' => esc_html__( 'Select sorting order.', 'wiloke-grid' )
					),

					array(
						'type' => 'dropdown',
						'heading' => esc_html__( 'Click Action', 'wiloke-grid' ),
						'param_name' => 'click_action',
						'value' => array(
							esc_html__( 'Link to item', 'wiloke-grid' ) => 'link_to_item',
							esc_html__( 'Open in Popup', 'wiloke-grid' ) => 'wiloke_grid_magnific_popup'
						),
						'default'	=> 'link_to_item',
						'dependency' => array(
							'element' => 'wil_post_type',
							'value_not_equal_to' => array( 'post')
						),
					),

		            array(
						'type' => 'dropdown',
						'heading' => esc_html__( 'Display Style', 'wiloke-grid' ),
						'param_name' => 'wil_display',
						'value' => array(
							esc_html__( 'Show All', 'wiloke-grid' ) => 'all',
							esc_html__( 'Load More Button', 'wiloke-grid' ) => 'btn',
							esc_html__( 'Lazy loading', 'wiloke-grid' ) => 'lazy',
						),
						'description' => esc_html__( 'Select display style for grid.', 'wiloke-grid' ),
						'dependency' => array(
							'element' => 'wil_post_type',
							'value_not_equal_to' => array( 'facebook')
						),
					),

					array(
						'type' => 'textfield',
						'heading' => esc_html__( 'Button Text', 'wiloke-grid' ),
						'param_name' => 'wil_btn_text',
						'value' => esc_html__( 'Load More', 'wiloke-grid' ),
						'dependency' => array(
							'element' => 'wil_display',
							'value' => array( 'btn' ),
						),
					),

					array(
						'type' 				=> 'textfield',
						'heading' 			=> esc_html__( 'Extra class name', 'wiloke-grid' ),
						'param_name' 		=> 'wil_class',
						'description' 		=> esc_html__( 'Style particular content element differently - add a class name and refer to it in custom CSS.', 'wiloke-grid' ),
					),

					// Design
					array(
		                "type" 				=> "wiloke_grid",
		                "heading" 			=> esc_html__("Design Layout", 'wiloke-grid'),
		                "param_name" 		=> "wil_layout",
		                "value" 			=> '',
		                'description' 		=> esc_html__( 'Click me to open popup design layout.', 'wiloke-grid' ),
		                'group' 			=> esc_html__( 'Design', 'wiloke-grid' ),
		            ),

		            array(
		                "type" 				=> "dropdown",
		                "heading" 			=> esc_html__("Item Style", 'wiloke-grid'),
		                "param_name" 		=> "wil_item",
		                "value" 			=> $this->itemStyle(),
		                'description' 		=> esc_html__( 'Create new template or modify selected. Predefined templates will be cloned.', 'wiloke-grid' ),
		                'group' 			=> esc_html__( 'Design', 'wiloke-grid' ),
		            ),

		            array(
		                "type" 				=> "dropdown",
		                "heading" 			=> esc_html__("Animation Scroll", 'wiloke-grid'),
		                "param_name" 		=> "wil_animation",
		                "value" 			=> $this->itemAnimation(),
		                'description' 		=> esc_html__( 'Animation when window scroll.', 'wiloke-grid' ),
		                'group' 			=> esc_html__( 'Design', 'wiloke-grid' ),
		            ),

		            array(
		                "type" 				=> "dropdown",
		                "heading" 			=> esc_html__("Hover Effect", 'wiloke-grid'),
		                "param_name" 		=> "wil_hover_effect",
		                "value" 			=> $this->itemHoverEffect(),
		                'group' 			=> esc_html__( 'Design', 'wiloke-grid' ),
		            ),
					//Filter tab
					array(
						'type' => 'dropdown',
						'heading' => esc_html__( 'Filter Style', 'wiloke-grid' ),
						'param_name' => 'wil_filter_style',
						'value' => $this->filtersStyle(),
						'dependency' => array(
							'element' => 'wil_show_filter',
							'value' => array( 'yes' ),
						),
						'group' => esc_html__( 'Filter', 'wiloke-grid' ),
						'description' => esc_html__( 'Select filter display style.', 'wiloke-grid' ),
					),

					array(
						'type' => 'textfield',
						'heading' => esc_html__( 'Default title', 'wiloke-grid' ),
						'param_name' => 'wil_filter_title',
						'value' => esc_html__( 'All', 'wiloke-grid' ),
						'description' => esc_html__( 'Enter default title for filter option display (empty: "All").', 'wiloke-grid' ),
						'dependency' => array(
							'element' => 'wil_show_filter',
							'value' => array( 'yes' ),
						),
						'group' => esc_html__( 'Filter', 'wiloke-grid' ),
					),
					array(
						'type' => 'dropdown',
						'heading' => esc_html__( 'Alignment', 'wiloke-grid' ),
						'param_name' => 'wil_filter_align',
						'value' => array(
							esc_html__( 'Left', 'wiloke-grid' ) => 'text-left',
							esc_html__( 'Center', 'wiloke-grid' ) => 'text-center',
							esc_html__( 'Right', 'wiloke-grid' ) => 'text-right',
						),
						'dependency' => array(
							'element' => 'wil_show_filter',
							'value' => array( 'yes' ),
						),
						'group' => esc_html__( 'Filter', 'wiloke-grid' ),
						'description' => esc_html__( 'Select filter alignment.', 'wiloke-grid' ),
					),

					// Css Editor
					array(
						'type' => 'css_editor',
						'heading' => esc_html__( 'CSS box', 'wiloke-grid' ),
						'param_name' => 'wil_css',
						'group' => esc_html__( 'Design Options', 'wiloke-grid' ),
					),
		        )
		    ));
		}

		public function gridField( $settings, $value ) {

			$paramID = uniqid('wiloke_grid_');

			ob_start(); ?>

		   <div class="wil_param_grid" data-param-id="<?php echo esc_attr($paramID); ?>">
				<div class="wil-design--btn">
					<div class="wil-img--wrap">
						<img src="<?php echo plugin_dir_url( dirname( __FILE__ ) ) . 'admin/assets/img/design-cover.jpg'; ?>" alt="">
					</div>
					<span class="wil_param_grid__span wil-grid__btn btn--danger"><i class="fa fa-crop"></i> <?php echo esc_html__('Design', 'wiloke-grid') ?></span>
					<input name="<?php echo esc_attr( $settings['param_name'] ) ?>" class="wpb_vc_param_value wpb-textinput <?php echo esc_attr( $settings['param_name'] ) ?> <?php echo esc_attr( $settings['type'] ) ?>_field" type="hidden" value="<?php echo esc_attr( $value ) ?>" data-param-input="<?php echo esc_attr($paramID); ?>" />
				</div>
		 	</div>

		 	<?php 

		 		$output = ob_get_clean();

		 	return $output;
		}

		public function checkboxsField( $settings, $value ) {

			$output = '';

			if ( is_array( $value ) ) {
				$value = '';
			}

			$output .='<div class="wil_param_checkbox" data-value="'. $value .'">';

			$current_value = strlen( $value ) > 0 ? explode( ',', $value ) : array();

			$values = isset( $settings['value'] ) && is_array( $settings['value'] ) ? $settings['value'] : array( esc_html__( 'Yes', 'wiloke-grid' ) => 'true' );

			if ( ! empty( $values ) ) {

				foreach ( $values as $label => $v ) {

					$checked = count( $current_value ) > 0 && in_array( $v, $current_value ) ? ' checked' : '';

					$output .= ' <label class="vc_checkbox-label"><input id="'
					           . $settings['param_name'] . '-' . $v . '" value="'
					           . $v . '" class="wpb_vc_param_value '
					           . $settings['param_name'] . ' ' . $settings['type'] . '" type="checkbox" name="'
					           . $settings['param_name'] . '"'
					           . $checked . '> ' . $label . '</label>';
				}
			}

			$output .='</div>';

			return $output;
		}

		public function template() { 

			$templates = get_option('wiloke_grid_template', false); ?>
			
			<script type="text/html" id="wiloke-grid-tmpl">

				<!-- popup HTML -->
				<!-- wil-popup -->
				<!-- Styles Popup -->
				<div id="wil-styles-popup" class="wil-popup">

					<div class="wil-table">
						<div class="wil-table__cell">

							<!-- wil-popup--modal -->
							<div class="wil-popup--modal">

								<!-- Header -->
								<header class="wil-popup--header">
									<h4 class="wil-popup--title fl"><i class="fa fa-photo"></i> Styles</h4>
									<a href="#" class="wil-popup--close fr">
										<i class="fa fa-times"></i>
									</a>
								</header>
								<!-- /Header -->

								<!-- Content -->
								<div class="wil-popup--content">
									<ul class="wil-style">
										<li class="wil-style--item <%= (style == 'masonry') ? 'active' : '' %>" data-style="masonry">
											<span class="wil-style--title">Masonry</span>
											<div class="wil-style--thumb">
												<img src="<?php echo plugin_dir_url( dirname( __FILE__ ) ) . 'admin/assets/img/masonry.jpg'; ?>" alt="">
											</div>
										</li>
										<li class="wil-style--item <%= (style == 'grid') ? 'active' : '' %>" data-style="grid">
											<span class="wil-style--title">Grid</span>
											<div class="wil-style--thumb">
												<img src="<?php echo plugin_dir_url( dirname( __FILE__ ) ) . 'admin/assets/img/grid.jpg'; ?>" alt="">
											</div>
										</li>
										<li class="wil-style--item <%= (style == 'creative') ? 'active' : '' %>" data-style="creative">
											<span class="wil-style--title">Creative</span>
											<div class="wil-style--thumb">
												<img src="<?php echo plugin_dir_url( dirname( __FILE__ ) ) . 'admin/assets/img/creative.jpg'; ?>" alt="">
											</div>
										</li>
									</ul>
								</div>
								<!-- /Content -->

								<!-- footer -->
								<footer class="wil-popup--footer">
									<div class="form__submit fr">
										<button id="wil-cancel-change-style" data-style="<%= style %>" class="wil-grid__btn btn--danger wil-close-popup"><i class="fa fa-times"></i> <?php echo esc_html__('Cancel', 'wiloke-grid') ?></button>
										<button id="wil-change-style" class="wil-grid__btn btn--primary"><i class="fa fa-save"></i> <?php echo esc_html__('OK', 'wiloke-grid') ?></button>
									</div>
								</footer>
								<!-- /footer -->

							</div>
							<!-- /wil-popup--modal -->

						</div>
					</div>
						
				</div>
				<!-- /Styles Popup -->
				<!-- /wil-popup -->

				<!-- wil-popup -->
				<!-- Save as Template Popup -->
				<div id="wil-save-template-popup" class="wil-popup">

					<div class="wil-table">
						<div class="wil-table__cell">

							<!-- wil-popup--modal -->
							<div class="wil-popup--modal">

								<!-- Header -->
								<header class="wil-popup--header">
									<h4 class="wil-popup--title fl"><i class="fa fa-hdd-o"></i> Save as Template</h4>
									<a href="#" class="wil-popup--close fr">
										<i class="fa fa-times"></i>
									</a>
								</header>
								<!-- /Header -->

								<!-- Content -->
								<div class="wil-popup--content">
									<div class="form__field mt-20 mb-30">
										<span class="form__field-label"><?php echo esc_html__('Template name', 'wiloke-grid') ?></span>
										<input type="text" class="form__field-control" value="">
									</div>
								</div>
								<!-- /Content -->

								<!-- footer -->
								<footer class="wil-popup--footer">
									<div class="form__submit fr">
										<button class="wil-grid__btn btn--danger wil-close-popup"><i class="fa fa-times"></i> <?php echo esc_html__('Cancel', 'wiloke-grid') ?></button>
										<button class="wil-grid__btn btn--primary"><i class="fa fa-save"></i> <?php echo esc_html__('Save', 'wiloke-grid') ?></button>
									</div>
								</footer>
								<!-- /footer -->

							</div>
							<!-- /wil-popup--modal -->

						</div>
					</div>
						
				</div>
				<!-- /Save as Template Popup -->
				<!-- /wil-popup -->

				<!-- wil-popup -->
				<!-- template Popup -->
				<div id="wil-template-popup" class="wil-popup">

					<div class="wil-table">
						<div class="wil-table__cell">

							<!-- wil-popup--modal -->
							<div class="wil-popup--modal">

								<!-- Header -->
								<header class="wil-popup--header">
									<h4 class="wil-popup--title fl"><i class="fa fa-files-o"></i> Templates</h4>
									<a href="#" class="wil-popup--close fr">
										<i class="fa fa-times"></i>
									</a>
								</header>
								<!-- /Header -->

								<!-- Content -->
								<div class="wil-popup--content">
									<ul class="wil-template wil-grid-admin__bottom wil-grid-admin__template" data-nonce="<?php echo esc_attr(wp_create_nonce('wiloke_grid_nonce_tmpl')); ?>">
										<?php  if ($templates) : ?>
											<?php foreach ($templates as $template) : ?>
				        			 		<li class="wil-template--item">
												<a class="wil-template--item-title" href="#" data-id="<?php echo esc_attr($template['id']) ?>"><?php echo esc_html($template['name']) ?></a>
												<div class="wil-template--item-remove" title="Remove" data-id="<?php echo esc_attr($template['id']) ?>">
													<i class="fa fa-times"></i>
												</div>
											</li>
					        				<?php endforeach; ?>
					        			<?php else: ?>
					        				<li class="wil-empty-template"><?php esc_html_e('You do not have any template yet', 'wiloke-grid'); ?></li>
					        			<?php endif; ?>
						        	</ul>
								</div>
								<!-- /Content -->

								<!-- footer -->
								<footer class="wil-popup--footer">
									<div class="form__submit fr">
										<button class="wil-grid__btn btn--danger wil-close-popup"><i class="fa fa-times"></i> <?php echo esc_html__('Cancel', 'wiloke-grid') ?></button>
										<button class="wil-grid__btn btn--primary"><i class="fa fa-save"></i> <?php echo esc_html__('OK', 'wiloke-grid') ?></button>
									</div>
								</footer>
								<!-- /footer -->

							</div>
							<!-- /wil-popup--modal -->

						</div>
					</div>
						
				</div>
				<!-- /template Popup -->
				<!-- /wil-popup -->
				<!-- /popup HTML -->

		        <div class="wil-grid-admin__inner">
		            
		            <div class="wil-grid-admin__nav">
		                <div class="nav__item nav__item--add"><span><?php echo esc_html__('Add Screen', 'wiloke-grid') ?></span></div>
		            </div>
		        
		            <div class="wil-grid-admin__panel"></div>

		        </div>

		        <div class="wil-grid-admin__bottom wil-grid-admin__template" data-nonce="<?php echo esc_attr(wp_create_nonce('wiloke_grid_nonce_tmpl')); ?>">

		        	<?php 
			        	if ($templates) {
			        		foreach ($templates as $template) : ?>
			        			<div class="bottom__item">
		        			 		<span class="bottom__item-text template__item" data-id="<?php echo esc_attr($template['id']) ?>"><?php echo esc_html($template['name']) ?></span>
		        			 		<span class="template__item-remove" data-id="<?php echo esc_attr($template['id']) ?>"></span>
	        			 		</div>
		        			<?php endforeach;
			        	}
		        	?>
		        </div>

		        <div class="wil-grid-admin__actions">
		            <button data-popup="wil-save-template-popup" class="wil-grid__btn btn-save-tmpl fl"><i class="fa fa-hdd-o"></i> <?php echo esc_html__('Save Template', 'wiloke-grid') ?></button>
		            <button data-popup="wil-template-popup" class="wil-grid__btn btn-tmpl fl"><i class="fa fa-files-o"></i> <?php echo esc_html__('Templates', 'wiloke-grid') ?></button>
		            <button data-popup="wil-styles-popup" class="wil-grid__btn btn-style fl"><i class="fa fa-photo"></i> <?php echo esc_html__('Style', 'wiloke-grid') ?></button>
		            <button class="wil-grid__btn btn--primary btn-save fr"><i class="fa fa-save"></i> <?php echo esc_html__('Save', 'wiloke-grid') ?></button>
		            <button class="wil-grid__btn btn--danger btn-cancel fr"><i class="fa fa-times"></i> <?php echo esc_html__('Cancel', 'wiloke-grid') ?></button>
		        </div>

			</script>

			<script type="text/html" id="wiloke-grid-packery-tmpl">

				<div class="panel__item-form">
    
                    <div class="form__field">
                        <span class="form__field-label"><?php echo esc_html__('Device', 'wiloke-grid') ?></span>
                        <input type="number" class="form__field-control input-device" value="<%= device %>">
                    </div>
    
                    <div class="form__field">
                        <span class="form__field-label"><?php echo esc_html__('Columns', 'wiloke-grid') ?></span>
                        <select class="form__field-control input-columns">
                            <option value="1" <%= (column == 2) ? 'selected' : '' %> >1</option>
		 					<option value="2" <%= (column == 3) ? 'selected' : '' %> >2</option>
		 					<option value="3" <%= (column == 3) ? 'selected' : '' %> >3</option>
		 					<option value="4" <%= (column == 4) ? 'selected' : '' %> >4</option>
		 					<option value="5" <%= (column == 5) ? 'selected' : '' %> >5</option>
		 					<option value="6" <%= (column == 6) ? 'selected' : '' %> >6</option>
                        </select>
                    </div>

                    <div class="form__field">
                        <span class="form__field-label"><?php echo esc_html__('Horizontal', 'wiloke-grid') ?></span>
                        <input type="number" class="form__field-control input-horizontal" value="<%= horizontal %>">
                    </div>
    
                    <div class="form__field">
                        <span class="form__field-label"><?php echo esc_html__('Vertical', 'wiloke-grid') ?></span>
                        <input type="number" class="form__field-control input-vertical" value="<%= vertical %>">
                    </div>
    
                    <div class="form__field">
                        <span class="form__field-label"><?php echo esc_html__('Items', 'wiloke-grid') ?></span>
                        <input type="number" class="form__field-control input-items" value="<%= items %>">
                    </div>
    
                </div>
    
                <div class="panel__item-packery">
                	<div class="wil-packery"></div>
                </div>

			</script>

			<script type="text/html" id="wiloke-grid-popup-tmpl">

		        <div class="wil-table">
		            <div class="wil-table__cell">

		                <div class="popup__inner">

		                    <h4 class="popup__title"><?php echo esc_html__('Save this template', 'wiloke-grid') ?></h4>

		                    <div class="popup__form">

		                        <div class="form__field">
		                            <span class="form__field-label"><?php echo esc_html__('Name', 'wiloke-grid') ?></span>
		                            <input type="text" class="form__field-control input-name" data-nonce="<?php echo esc_attr(wp_create_nonce('wiloke_grid_nonce_create_tmpl')) ?>">
		                        </div>

		                        <div class="form__submit">
		                            <button class="wil-grid__btn btn--primary btn-save"><i class="fa fa-save"></i> <?php echo esc_html__('Save', 'wiloke-grid') ?></button>
		                            <button class="wil-grid__btn btn--danger btn-cancel wil-close-popup"><i class="fa fa-times"></i> <?php echo esc_html__('Cancel', 'wiloke-grid') ?></button>
		                        </div>
		                    </div>

		                </div>

		            </div>
		        </div>

			</script>

			<?php 
		}
	}

	new WilokeGridAdmin();
}