// Wiloke Grid
(function(factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        // CommonJS
        factory(require('jquery'));
    } else {
        // Browser globals
        factory(jQuery);
    }
}(function($) {

    'use strict';

    var WilokeGrid = {
        View: {}
    }

    WilokeGrid.Guid = function() {
        var d = new Date().getTime();
        var str = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
            var r = (d + Math.random() * 16) % 16 | 0;
            d = Math.floor(d / 16);
            return (c == 'x' ? r : (r & 0x3 | 0x8)).toString(16);
        });

        var uuid = str.substring(0, 12);

        return uuid;
    }

    WilokeGrid.Default = function() {

        var model = {
            id: WilokeGrid.Guid(),
            device: 320,
            column: 4,
            vertical: 10,
            horizontal: 10,
            items: 8,
            // more: 8,
            grid: '[{"width":1,"height":1},{"width":1,"height":1},{"width":1,"height":1},{"width":1,"height":1},{"width":1,"height":1},{"width":1,"height":1},{"width":1,"height":1},{"width":1,"height":1}]'
        }

        return model;
    }

    WilokeGrid.Model = Backbone.Model.extend({

        defaults: function() {
            return WilokeGrid.Default();
        },

        sync: function(method, model, options) {
            
        },
    })

    WilokeGrid.Collection = Backbone.Collection.extend({
        model: WilokeGrid.Model
    })

    WilokeGrid.View.Tab = Backbone.View.extend({
        tagName: 'div',
        className: 'nav__item nav__item--device active',
        template: _.template(
            '<span class="nav__item-text"><%= device %>px</span>\
            <span class="nav__item-remove"><span></span></span>'
        ),

        initialize: function() {
            this.render();
            this.listenTo(this.model, 'change:device', this.device);
            this.listenTo(this.model, 'destroy', this.destroy);
        },

        events: {
            "click .nav__item-text": "activeTab",
            "click .nav__item-remove": "removeTab"
        },

        activeTab: function(event) {

            var self = $(event.target),
                id = this.model.get('id');

            if (!self.parent().hasClass('active')) {
                $('.nav__item.active').removeClass('active');
                $('.panel__item.active').removeClass('active');
                $('#' + id).addClass('active');
                self.parent().addClass('active');
            }
        },

        removeTab: function(event) {
            event.preventDefault();

            var choice = confirm('Are you sure you want to delete this item?');

            if (choice) {
                this.model.destroy();
            }
        },

        destroy: function() {
            this.unbind();
            this.remove();
        },

        device: function() {
            this.$el.find('.nav__item-text').text(this.model.get('device') + 'px')
        },

        render: function() {
            this.$el.append(this.template(this.model.toJSON()));
            $('.nav__item.active').removeClass('active');
        }

    })

    WilokeGrid.View.SingleTab = Backbone.View.extend({
        tagName: 'div',
        className: 'panel__item active',

        initialize: function() {
            this.render();
            this.listenTo(this.model, 'destroy', this.destroy);
        },

        destroy: function() {
            this.remove();
            this.unbind();
        },

        render: function() {
            $('.panel__item.active').removeClass('active');
            this.$el.attr('id', this.model.get('id'));
        }
    })

    WilokeGrid.View.Packery = Backbone.View.extend({
        tagName: 'div',
        className: 'panel__item-content',
        packery: !1,
        template: _.template($('#wiloke-grid-packery-tmpl').html()),
        events: {
            "change .input-device": "changeDevice",
            "change .input-columns": "changeColumn",
            "change .input-vertical": "changeVertical",
            "change .input-horizontal": "changeHorizontal",
            "change .input-items": "changeItems"
        },

        initialize: function() {
            this.render();
            this.createPackery();
            this.listenTo(this.model, 'destroy', this.destroy);
            this.listenTo(this.model, 'change:style', this.changeStyle);
        },

        createPackery: function() {

            var _this = this,
                option = this.model.toJSON();

            this.packery = new WilokePackery(this.$el.find('.wil-packery'), {
                style: option.style,
                column: option.column,
                vertical: option.vertical,
                horizontal: option.horizontal,
                grid: JSON.parse(option.grid),
                onResizableEnd: function() {
                    _this.model.set('grid', _this.packery.getItems(false));
                    _this.model.save();
                }
            });
        },

        changeDevice: function(event) {

            var self = $(event.target),
                device = parseInt(self.val());

            if (device >= 0) {
                this.model.set('device', device);
                this.model.save();
            }
        },


        changeVertical: function() {

            var self = $(event.target),
                val = parseInt(self.val());

            if (val >= 0 && this.packery) {
                this.packery.changeVertical(val);
                this.model.set('vertical', val);
                this.model.save();
            }
        },

        changeHorizontal: function() {

            var self = $(event.target),
                val = parseInt(self.val());

            if (val >= 0 && this.packery) {
                this.packery.changeHorizontal(val);
                this.model.set('horizontal', val);
                this.model.save();
            }
        },

        changeColumn: function(event) {

            var self = $(event.target),
                val = parseInt(self.val());

            if (val > 0 && this.packery) {
                this.packery.changeColumn(val);
                this.model.set('column', val);
                this.model.set('grid', this.packery.getItems(false));
                this.model.save();
            }
        },

        changeItems: function(event) {

            var self = $(event.target),
                oldItems = this.model.get('items'),
                newItems = parseInt(self.val());

            if (newItems >= 0 && this.packery) {


                if (newItems > oldItems) {
                    this.packery.addItems(newItems - oldItems);
                } else {
                    this.packery.removeItems(oldItems - newItems);
                }

                this.model.set('items', newItems);

                this.model.set('grid', this.packery.getItems(false));

                this.model.save();
                
            }
        },

        changeStyle: function() {

            var model = this.model.toJSON();

            this.packery.changeStyle(model.style);

            if (model.style == "creative") {
                this.packery.enableDraggie('enable');
            } else {
                this.packery.enableDraggie('disable');
            }
        },

        destroy: function() {
            this.unbind();
            this.remove();
        },

        render: function() {

            var model = this.model.toJSON();

            this.$el.html(this.template(model));
        }

    });

    WilokeGrid.View.Template = Backbone.View.extend({
        tagName: 'div',
        className: 'wil-grid-admin__popup',
        template: _.template($('#wiloke-grid-popup-tmpl').html()),

        initialize: function() {
            this.render();
        },

        events: {
            "click .btn-cancel": "cancel",
            "click .btn-save": "save",
        },

        save: function(event) {
            event.preventDefault();

            var input, name, nonce, param, id, _this = this, self = $(event.currentTarget);

            input = this.$el.find('.input-name');
            name = input.val();
            nonce = input.data('nonce');
            id = WilokeGrid.Guid();

            if( !self.parent().hasClass('ajax') && name != '' ) {
                param = {
                    action: 'wil_grid_save_tmpl',
                    security: nonce,
                    data: {
                        id: id,
                        name: name,
                        models: this.model
                    }
                };
 // console.log(param);
                $.ajax({
                    url: ajaxurl,
                    data: param,
                    type: 'POST',
                    beforeSend: function() {
                        self.parent().addClass('ajax');
                    }
                })
                .done(function(response) {

                    if(response['status'] == 200) {

                        var $parent = $('.wil-grid-admin__template'),
                            html = '<li class="wil-template--item">';
                            html += '<a class="wil-template--item-title" href="#" data-id="'+ id +'">'+ name +'</a>';
                            html += '<div class="wil-template--item-remove" title="Remove" data-id="'+ id +'"><i class="fa fa-times"></i></div>';
                            html +='</li>';

                        $parent.find('.wil-empty-template').remove();
                        $('.wil-grid-admin__template').append(html);

                        self.parent().removeClass('ajax');

                        _this.cancel();
                    }

                });
            }
            
        },

        cancel:function() {
            this.unbind();
            this.remove();
        },

        render: function() {
            this.$el.append(this.template());
        }

    });

    WilokeGrid.View.Panel = Backbone.View.extend({
        tagName: 'div',
        className: 'wil-grid-admin',
        collection: new WilokeGrid.Collection(),
        template: _.template( $('#wiloke-grid-tmpl').html() ),

        initialize: function() {
            this.render();
            this.readModels(this.model.devices);
        },

        events: {
            "click .nav__item--add": "addElement",
            "click .btn-save": "save",
            "click .btn-cancel": "cancel",
            "click .item-tab": "changeTab",
            'click .btn-style': 'showStyle',
            // "click .style__item": "changeStyle",
            "click .wil-style--item": "changeStyle",
            "click #wil-change-style": "changedStyle",
            "click #wil-cancel-change-style": "resetEmulateActivation",
            "click .btn-tmpl": "showTemplate",
            "click .wil-template--item-title": "addTemplate",
            "click .wil-template--item-remove": "removeTemplate",
            "click .btn-save-tmpl": "showPopupTemplete",
            "click .wil-popup--close, .wil-close-popup": "closePopup"
        },

        readModels: function(models) {
            
            if (models) {

                this.destroyModels();

                _.each(models, function(model) {

                    if( _.isArray(model['grid']) || _.isObject(model['grid'])  ) {
                        model['grid'] = JSON.stringify(model['grid']);
                    }

                    model['style'] = this.model.style;

                    var _model = this.collection.create(model);

                    this.createElement(_model);

                }, this);
            }
        },

        addElement: function(event) {

            var model = this.collection.create({
                style: this.model.style
            });

            this.createElement(model);
        },

        createElement: function(model) {
            var viewTab, viewSingleTab, viewPackery;

            viewTab = new WilokeGrid.View.Tab({ model: model });
            viewSingleTab = new WilokeGrid.View.SingleTab({ model: model });
            viewPackery = new WilokeGrid.View.Packery({ model: model });

            viewSingleTab.$el.append(viewPackery.$el);

            this.$el.find('.wil-grid-admin__nav').prepend(viewTab.$el);
            this.$el.find('.wil-grid-admin__panel').append(viewSingleTab.$el);
        },

        getModels: function() {

            var models = this.collection.toJSON();

            if( !_.isEmpty(models) ) {

                var _models = {
                        style: this.model.style,
                        devices: {}
                    };

                _.each(models, function(model) {
                    _models.devices[model.device] = {
                        device: model.device,
                        column: model.column,
                        vertical: model.vertical,
                        horizontal: model.horizontal,
                        items: model.items,
                        grid: JSON.parse(model.grid)
                    };
                });

                return _models;
            }

            return '';
        },

        parser: function() {

            var models = this.getModels();

            if( !_.isEmpty(models) ) {
                return encodeURIComponent(JSON.stringify(models));
            }

            return '';
        },

        showStyle: function(event) {
            event.preventDefault();
            var self = $(event.target);
            
            if( self.hasClass('active') ) {
                self.removeClass('active');
                // this.$el.find('.wil-grid-admin__style').removeClass('active');
                this.$el.find('#'+self.data('popup')).removeClass('active');
            } else {
                // this.$el.find('.wil-grid-admin__actions .wil-grid__btn, .wil-grid-admin__bottom').removeClass('active');
                this.$el.find('#'+self.data('popup')).addClass('active');
                self.addClass('active');
                // this.$el.find('.wil-grid-admin__style').addClass('active');
            }
            
        },

        resetEmulateActivation: function(event){
            var self = $(event.target),
                style = self.data('style');

            this.$el.find('.wil-style--item').removeClass('active');
            this.$el.find('.wil-style--item[data-style="'+style+'"]').addClass('active');
        },

        changedStyle: function(event){
            var self = $(event.target),
                val = this.$el.find('.wil-style--item.active').data().style;

            if ( val != this.model.style ) {
                _.each(this.collection.models, function(model, el) {
                    model.set('style', val);
                });
                this.model.style = val;
            }

            this.$el.find('.wil-popup, .wil-grid__btn').removeClass('active');
        },

        changeStyle: function(event) {
            event.preventDefault();
            this.$el.find('.wil-style--item').removeClass('active');
            $(event.currentTarget).addClass('active');
        },

        showTemplate: function(event) {
            event.preventDefault();

            var self = $(event.target);

            if( self.hasClass('active') ) {
                self.removeClass('active');
                // this.$el.find('.wil-grid-admin__template').removeClass('active');
                this.$el.find('#'+self.data('popup')).removeClass('active');
            } else {
                // this.$el.find('.wil-grid-admin__actions .wil-grid__btn, .wil-grid-admin__bottom').removeClass('active'); 
                this.$el.find('.wil-popup').removeClass('active');

                self.addClass('active');
                // this.$el.find('.wil-grid-admin__template').addClass('active');
                this.$el.find('#'+self.data('popup')).addClass('active');
            }
        },

        addTemplate: function(event) {
            event.preventDefault();

            var _this   = this,
                self    = $(event.currentTarget),
                val     = self.data().id,
                nonce   = self.closest('.wil-grid-admin__template').data().nonce;

            if( !self.parent().hasClass('ajax') && val !='' ) {

                var param = {
                    action: 'wil_grid_get_tmpl',
                    security: nonce,
                    key: val
                };

                $.ajax({
                    url: ajaxurl,
                    data: param,
                    type: 'POST',
                    beforeSend: function() {
                        self.parent().addClass('ajax');
                    }
                })
                .done(function(response) {

                    if(response['status'] == 200) {
                        _this.model.style = response['models']['style'];
                        var models =  response['models']['devices'];
                        _this.readModels(models);       
                    }

                    self.parent().removeClass('ajax');

                    _this.$el.find('.wil-popup').removeClass('active');
                });

            }
            
        },

        removeTemplate: function(event) {
            event.preventDefault();

            var _this = this,
                self = $(event.currentTarget),
                val = self.data().id,
                nonce = self.closest('.wil-grid-admin__template').data().nonce;

             if( !self.parent().hasClass('ajax') && val !='' ) {

                var param = {
                    action: 'wil_grid_remove_tmpl',
                    security: nonce,
                    key: val
                };

                $.ajax({
                    url: ajaxurl,
                    data: param,
                    type: 'POST',
                    beforeSend: function() {
                        self.parent().addClass('ajax');
                    }
                })
                .done(function(response) {

                    if(response['status'] == 200) {
                        self.parent().remove();
                    }

                    self.parent().removeClass('ajax');

                });
            }
        },

        showPopupTemplete: function(event) {
            event.preventDefault();

            var model, view, self = $(event.target);

            model = this.getModels();
            
            view = new WilokeGrid.View.Template({ model: model });
            this.$el.find('.wil-grid-admin__actions .wil-grid__btn, .wil-grid-admin__bottom').removeClass('active');
            // this.$el.find('.wil-popup').removeClass('active');
            $('body').append(view.$el);
        },

        closePopup: function(event){
            event.preventDefault();
            var self = $(event.target), popupID = self.closest('.wil-popup').attr('id');
            self.closest('.wil-popup').removeClass('active');
            this.$el.find('[data-popup="'+popupID+'"]').removeClass('active');
        },

        save: function(event) {
            event.preventDefault();
            
            if ($('[data-param-input="' + this.model.paramID + '"]').length) {
                $('[data-param-input="' + this.model.paramID + '"]').val(this.parser());
            }

            this.cancel(event);
        },

        cancel: function(event) {
            event.preventDefault();
            $('body').removeClass('overflow-hidden');
            this.destroyModels();
            this.unbind();
            this.remove();
        },

        destroyModels: function() {

            if(this.collection.models) {

                _.each(_.clone(this.collection.models), function(model) {
                    model.destroy();
                });
            }
        },

        render: function() {
            $('body').append(this.$el.append(this.template(this.model)));
        },

    });

    window.WilokeGrid = WilokeGrid;

}));
