// Wiloke Packery
(function(factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        // CommonJS
        factory(require('jquery'));
    } else {
        // Browser globals
        factory(jQuery);
    }
}(function($) {
    'use strict';

    var WilokePackery = function(el, options) {
        this.packery = el,
        this.options = $.extend({}, WilokePackery.default, options);
        this.style = this.options.style;
        this.draggie = this.options.draggie;
        this.grid = this.options.grid;
        this.gridSize = this.options.gridSize;
        this.gridItem = this.options.gridItem;
        this.horizontal = this.options.horizontal;
        this.vertical = this.options.vertical;
        this.column = this.options.column;
        
        setTimeout($.proxy(function() {
            this.init();
        }, this), 100);
    }

    WilokePackery.default = {
        style: 'creative',
        draggie: true,
        gridSize: '.grid-sizer',
        gridItem: '.grid-item',
        column: 5,
        vertical: 10,
        horizontal: 10,
        onResizableEnd: false,
        grid: [{width: 2, height: 2}, {width: 1, height: 1}, {width: 1, height: 2}, {width: 1, height: 1}, {width: 1, height: 1}, {width: 1, height: 1}, {width: 1, height: 1}, {width: 1, height: 1}]
    }

    WilokePackery.prototype = {

        init: function() {
            this.settings();
            this.createPackery();
            this.render();
            this.onResize();
        },

        createPackery: function() {

            this.packery.packery({
                itemSelector: this.gridItem,
                columnWidth: this.gridSize
            });

            this.dragItemPositioned();
        },

        settings: function() {

            this.packery.addClass('wil-packery-' + this.style);

            if(this.style == 'creative' ) {
                this.packery.addClass('wil-packery-draggie-enable');
            } else {
                this.packery.addClass('wil-packery-draggie-disable');
            }

            this.containerSize();

            this.createSize();
        },

        containerSize: function() {
            this.packery.css({
                'margin-left': -this.horizontal/2,
                'margin-right': -this.horizontal/2
            });
        },

        draggabilly: function(item) {

            item.draggabilly({
                handle: '.handle'
            });

            var draggie = item.draggabilly('enable');
            this.packery.packery( 'bindDraggabillyEvents', draggie.data('draggabilly'));
        },

        dragItemPositioned: function() {
            this.packery.on( 'dragItemPositioned', $.proxy(this.orderItems, this) );
        },

        orderItems: function() {

            if(this.draggie) {

                var _this = this,
                    itemElems = _this.packery.packery('getItemElements');

                $(itemElems).each( function( index, itemElem ) {

                    var self = $(itemElem),
                        width = parseInt(self.attr('data-width')),
                        height = parseInt(self.attr('data-height'));
                    
                    _this.grid[index]['width'] = width;
                    _this.grid[index]['height'] = height;

                    self.attr('data-order', index);

                });

            }

        },

        getSize: function() {

            var containerSize = this.packery.innerWidth();

            return Math.floor(containerSize/this.column);
        },

        createSize: function() {
            this.packery.append('<div class="'+ this.gridSize.replace('.', '') +'"></div>');
            this.setSize();
        },

        setSize: function() {
            var size = this.getSize();
            this.packery.find(this.gridSize).css('width', size);
        },

        refresh: function() {

            this.setSize();

            _.each(this.packery.find(this.gridItem), function(item, index) {
                
                var self = $(item);

                this.setItemSize(self);

            }, this);

            this.packery.packery();

        },

        createItem: function(data, order) {
            var item = $('<div class="'+ this.gridItem.replace('.', '') +'" data-width="'+ data.width +'" data-height="'+ data.height +'" data-order="'+ order +'">\
                <div class="inner">\
                    <div class="splitter right"></div>\
                    <div class="splitter bottom"></div>\
                    <div class="handle"></div>\
                </div>\
            </div>');

            return item;
        },

        appended: function(data, order) {
            var size = this.getSize(),
                item = this.createItem(data, order);

            this.setItemSize(item);
            this.draggabilly(item);
            this.resizableItem(item);
            this.packery.append(item).packery( 'appended', item ).packery();
        },

        addItems: function(countItem) {

            if( countItem > 0 ) {

                var shape = { width: 1, height: 1 },
                    countItems = this.packery.find(this.gridItem).length;

                for (var i = 0; i < countItem; i++) {
                    this.grid.push(shape);
                    this.appended(shape, countItems + i);
                }

            }
        },

        removeItems: function(countRemove) {

            if(countRemove > 0) {

                var countItem = this.grid.length,
                    traction = countItem - countRemove,
                    arr = [];

                if( traction < 0 ) {
                    traction = 0;
                    countRemove = countItem;
                }

                this.grid.splice(traction, countRemove);

                var itemElems = this.packery.packery('getItemElements');
                
                for (var i = 0; i < countRemove; i++) {
                    var itemRemove = $(itemElems[this.grid.length + i]);
                    this.packery.packery( 'remove', itemRemove ).packery();
                }
            }
        },

        setItemSize: function(item) {

            var size = this.getSize(),
                index = item.attr('data-order'),
                _size = size,
                _height = size;

            if(this.style == 'creative' ) {

                var dataWidth = parseInt(item.attr('data-width')),
                    dataHeight = parseInt(item.attr('data-height'));

                if( dataWidth > this.column )
                    dataWidth = this.column;

                if ( dataHeight > this.column ) 
                    dataHeight = this.column;

                _size = _size*dataWidth;

                _height = _height*dataHeight;

                item.attr('data-height', dataHeight);
                item.attr('data-width', dataWidth);

                this.grid[index] = {width: dataWidth, height: dataHeight};

            } else if( this.style == 'masonry') {
                _height = Math.floor((Math.random() * _height/2) + _height);
            }

            item.css({
                'width': _size,
                'height': _height,
                'padding-left': this.horizontal/2,
                'padding-right': this.horizontal/2,
                'padding-bottom': this.vertical/2,
                'padding-top': this.vertical/2,
            }).attr('data-column', this.column).attr('data-size', size);
        },

        resizableItem: function(item) {

            var _this = this;

            item.WilokeResizable({
                handleSelector: '.splitter',
                onDrag: function() {
                    setTimeout(function() {
                        _this.packery.packery();
                    }, 100);
                },
                onDragEnd: function(index, width, height) {
                    
                    var current = _this.grid[index];

                    if(current.width != width || current.height != height) {
                        _this.grid[index] = {width: width, height: height};

                        setTimeout(function() {
                            _this.refresh();
                        }, 100);

                        if (typeof _this.options.onResizableEnd === "function") {
                            _this.options.onResizableEnd.apply(_this);
                        }

                    } else {
                        setTimeout(function() {
                            _this.packery.packery();
                        }, 100);
                    }
                }
            });

        },

        render: function() {

            if( this.grid.length ) {

                var _this = this;

                $.each(_this.grid, function(index, data) {
                    _this.appended(data, index);
                });

                setTimeout(function() {
                    $(window).trigger('resize');
                }, 100);
            }

        },

        getItems: function(data) {
            if(data) {
                return this.grid;
            } else {
                return JSON.stringify(this.grid);
            }
        },

        changeColumn: function(column) {

            if(column > 0) {
                this.column = column;
                this.refresh();
            }
        },

        changeVertical: function(spacing) {
            if(spacing >= 0 && this.vertical != spacing) {
                this.vertical = spacing;
                this.refresh();
            }
        },

        changeHorizontal: function(spacing) {

            if(spacing >= 0 && this.horizontal != spacing) {
                this.horizontal = spacing;
                this.containerSize();
                this.refresh();
            }
        },

        changeStyle: function(style) {

            if( $.inArray(style, ['masonry', "grid", "creative"]) != -1 && this.style != style ) {
                this.style = style;
                this.packery.removeClass('wil-packery-grid wil-packery-creative wil-packery-masonry').addClass('wil-packery-' + this.style);
                this.refresh();
            }
        },

        enableDraggie: function(methods) {

            if(methods == 'disable') {
                this.packery.removeClass('wil-packery-draggie-enable').addClass('wil-packery-draggie-disable')
            } else {
                this.packery.removeClass('wil-packery-draggie-disable').addClass('wil-packery-draggie-enable')
            }
        },

        onResize: function() {

            var containerSize = this.packery.innerWidth();

            $(window).on('resize', $.proxy(function(event) {
                
                var _containerSize = this.packery.innerWidth();
                
                if( containerSize != _containerSize) {
                    this.refresh();
                }
            }, this));
        }
    }

    window.WilokePackery = WilokePackery;

    $.fn.WilokePackery = function(options) {

        return this.each(function(index, el) {
            var self = $(this),
                data = self.data('WilokePackery');

            if(!data) {
                data = new WilokePackery($(this), options);
                self.data('WilokePackery', data);
            }
        });
    }

}));