(function(factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        // CommonJS
        factory(require('jquery'));
    } else {
        // Browser globals
        factory(jQuery);
    }
}(function($) {

    'use strict';

    vc.atts.wiloke_grid = {

        init:function(param, $field) {
            
            $('.wil_param_grid', $field).on('click', function(event) {

                var self = $(this),
                    paramID = self.data('param-id'),
                    param = self.find('.wpb_vc_param_value'),
                    value = param.val();

                var model = {
                    paramID: paramID,
                    style: 'creative'
                }

                if( value !='' ) {
                    
                    var _model = JSON.parse(decodeURIComponent(value));

                    if(_.isObject(_model)) {
                        model = _.extend(model, _model);
                    }
                }

                $('body').addClass('overflow-hidden');

                new WilokeGrid.View.Panel({model: model});

            });
        }
    };

    vc.atts.wiloke_checkbox = {

        parse: function ( param ) {

            var arr, newValue;

            arr = [];

            newValue = '';

            $( 'input[name=' + param.param_name + ']', this.content() ).each( function () {

                var self;

                self = $( this );
                if ( this.checked ) {
                    arr.push( self.attr( 'value' ) );
                }

            });

            if ( 0 < arr.length ) {
                newValue = arr.join( ',' );
            }

            return newValue;
        }
    };

}));

function WilGridTaxonomiesCallBack() {
   
    var $taxonomy, $terms, $postType, exValue, taxValue, $ = jQuery;

    $postType = $(".wpb_vc_param_value[name=wil_post_type]", this.$content);

    $taxonomy = $(".wpb_vc_param_value[name=wil_taxonomy]", this.$content);

    $terms = $("[data-vc-shortcode-param-name=wil_terms]", this.$content);

    taxValue = $taxonomy.data().option;

    exValue = $terms.find('.wil_param_checkbox').data('value');
    
    if(typeof exValue == 'undefined') {
        exValue = '';
    }

    exValue = exValue.split(',');

    $postType.on('change', function(event) {
        var self = $(this),
            val = self.val();

        $.ajax({
            url: ajaxurl,
            dataType: 'json',
            data: {
                action: 'wil_grid_taxonomies',
                post_type: val
            },
        })
        .done(function(response) {

            $taxonomy.find('option').remove();
            $terms.find('label').remove();

            if(response) {

                $taxonomy.append('<option class="" value="">Select option ...</option>');

                $.each(response, function(key, value) {
                    
                    var selected = (taxValue == key) ? 'selected' : '';

                    $taxonomy.append('<option class="'+ key +'" value="'+ key + '" '+ selected +'>'+ value +'</option>');
                });

                var newTax = $taxonomy.val();

                $.ajax({
                    url: ajaxurl,
                    dataType: 'json',
                    data: {
                        action: 'wil_grid_terms',
                        taxonomy: newTax
                    },
                })
                .done(function(res) {

                    if(res) {

                        $.each(res, function(key, value) {

                            var checked = exValue.indexOf(key) != -1 ? 'checked' : '';

                            $terms.find('.wil_param_checkbox').append('<label class="vc_checkbox-label"><input id="wil_terms" value="'+ key +'" class="wpb_vc_param_value wil_terms wiloke_checkbox" type="checkbox" name="wil_terms" '+ checked +' data-dependent-set="true"> '+ value +'</label>');

                        });
                    }

                });
            }

        });
        
    }).trigger('change');
}

function WilGridTaxonomiesExcludeCallBack() {

    var $taxonomy, $terms, exValue, $ = jQuery, oSettings = {}, aNotEnqualTo=[];

    $terms = $("[data-vc-shortcode-param-name=wil_terms]", this.$content);

    $taxonomy = $(".wpb_vc_param_value[name=wil_taxonomy]", this.$content);

    $postType = $(".wpb_vc_param_value[name=wil_post_type]", this.$content);

    exValue = $terms.find('.wil_param_checkbox').data('value');
    
    oSettings = $terms.data('param_settings');
    aNotEnqualTo = oSettings.dependency.value_not_equal_to;
    
    if(typeof exValue == 'undefined') {
        exValue = '';
    }

    exValue = exValue.split(',');

    $taxonomy.on('change', function(event) {
        var self = $(this),
            val = self.val();

        $terms.find('label').remove();

        $.ajax({
            url: ajaxurl,
            dataType: 'json',
            data: {
                action: 'wil_grid_terms',
                taxonomy: val
            },
        })
        .done(function(response) {

            if(response) {

                $.each(response, function(key, value) {
                    
                    var checked = exValue.indexOf(key) != -1 ? 'checked' : '';

                    $terms.find('.wil_param_checkbox').append('<label class="vc_checkbox-label"><input id="wil_terms" value="'+ key +'" class="wpb_vc_param_value wil_terms wiloke_checkbox" type="checkbox" name="wil_terms" '+ checked +' data-dependent-set="true"> '+ value +'</label>');

                });
            }

        });
        
    }).trigger('change');

    if ( typeof aNotEnqualTo !== 'undefined' ){
        $postType.on('change', function(event) {
            if ( aNotEnqualTo.indexOf($(event.currentTarget).val()) !== -1 ){
                $terms.addClass('hidden');
            }else{
                $terms.removeClass('hidden');
            }
        })
    }
}