(function(factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        // CommonJS
        factory(require('jquery'));
    } else {
        // Browser globals
        factory(jQuery);
    }
}(function($) {
	'use strict';

	var WilokeIsotope = function(el, options) {
		this.$el = el;
		this.options = $.extend({}, WilokeIsotope.default, options);
		this.styles = this.options.style,
		this.container = this.$el.find(this.options.container);
		this.filter = this.$el.find(this.options.filter);
		this.button = this.options.button;
		this.gridSize = this.options.gridSize;
		this.gridItem = this.options.gridItem;
		this.beforeSetItemElem =  this.options.beforeSetItemElem;
		this.afterSetItemElem = this.options.afterSetItemElem;
		this.init();
	}

	WilokeIsotope.default = {
		style: 'masonry',
		filter: '.wil_filter',
		button: '.wil_btn_more',
		container: '.wil_masonry',
		gridSize: '.grid-sizer',
		gridItem: '.grid-item',
		gridItemInner: '.grid-item__inner',
		isotopeTransition: {
			hiddenStyle: '',
			visibleStyle: '',
			transitionDuration: ''
		},
		beforeSetItemElem: false,
		afterSetItemElem: false,
		data: {
			type: 'post',
			taxonomy: '',
			terms: '',
			term: 0,
			order: 'DESC',
			orderby: 'date',
			item: 'default',
			display: 'all',
			shortcode: '',
			nonce: '',
			post__in: '',
			post__not_in: ''
		}
	}

	WilokeIsotope.prototype = {

		init: function() {
			this.createIsotope();
			this.afterCreateIsotope();
			this.resizeBound();
			this.filterItem();
			this.buttonClick();
			this.lazyLoad();
			this.onResize();
		},

		createIsotope: function() {
			
			var options = {
				itemSelector: this.gridItem,
				masonry: {
					columnWidth: this.gridSize
				},
			};

			if(typeof this.options.isotopeTransition['transitionDuration'] !== 'undefined' && this.options.isotopeTransition['transitionDuration'] !== '') {
				options['transitionDuration'] = this.options.isotopeTransition['transitionDuration'];
			}

			if(typeof this.options.isotopeTransition['hiddenStyle'] !== 'undefined' && this.options.isotopeTransition['hiddenStyle'] !== '') {
				options['hiddenStyle'] = this.isotopeTransition['hiddenStyle'];
			}

			if(typeof this.options.isotopeTransition['visibleStyle'] !== 'undefined' && this.options.isotopeTransition['visibleStyle'] !== '') {
				options['visibleStyle'] = this.options.isotopeTransition['visibleStyle'];
			}

			this.container.imagesLoaded($.proxy(function() {
				this.container.isotope(options);
			}, this));

		},

		resizeBound: function() {

			if( typeof Outlayer !== 'undefined' ) {
				
				$.extend(Outlayer.prototype, {

					resize: function() {

						if ( !this.isResizeBound ) {
							return;
						}

						this.layout();
					}
				});
			}
		},

		afterCreateIsotope: function() {
			var point = this.breakpoint();
			this.$el.addClass('wil_masonry-' + this.options.style);
			this.settings(point);
		},

		settings: function(point) {
			
			if( point ) {
				this.setContainerSize(point);
				this.refresh();
			}
		},

		setContainerSize: function(point) {

			this.container.css({
				'margin-left': -point.horizontal/2,
				'margin-right': -point.horizontal/2 
			});
		},

		getContainerSize: function() {
			return this.container.width();
		},

		setSize: function(size) {
			$(this.gridSize, this.container).css('width', size);
		},

		getSize: function(point) {
			return this.getContainerSize()/point.column;
		},

		getItemElems: function() {
			return this.container.find(this.gridItem);
		},

		refresh: function() {
			var point = this.breakpoint(),
				size = this.getSize(point),
				itemElems = this.getItemElems();
			this.setSize(size);
			this.setItemElems(itemElems, point, size);
		},

		setItemElems: function(itemElems, point, size) {
			var index = 0,
				_this = this;
			itemElems.each(function(i, itemElem) {
				var self = $(this);
				if(index > point.grid.length - 1) {
					index = 0;
				}
				_this.setItemElem(self, point, size, index);
				index ++;
			});
		},

		setItemElem: function(itemElem, point, size, index) {

			var sizes = point.grid[index],
				width = size,
				height = '';
	
			if(typeof this.beforeSetItemElem === 'function') {
				this.beforeSetItemElem.apply(this, [itemElem, point, size, index]);
			}

			if( this.styles == 'creative' ) {
				width = sizes.width * size;
				height = Math.floor(size) * sizes.height;
			} else if(this.styles == 'grid') {
				height = Math.floor(size);
			}

			itemElem.css({
				'width': width,
				'height': height,
			});

			this.setSpacing(itemElem, point, index);

			if(typeof this.afterSetItemElem === 'function') {
				this.afterSetItemElem.apply(this, [itemElem, point, size, index]);
			}
		},

		setSpacing: function(itemElem, point, index) {

			if(this.styles == 'creative' || this.styles == 'grid') {

				itemElem.find(this.options.gridItemInner).css({
					'position': 'absolute',
					'top': point.vertical/2,
					'bottom': point.vertical/2,
					'left': point.horizontal/2,
					'right': point.horizontal/2,
				});

			} else {

				itemElem.find(this.options.gridItemInner).css({
					'position': 'relative',
					'padding-top': point.vertical/2,
					'padding-bottom': point.vertical/2,
					'padding-left': point.horizontal/2,
					'padding-right': point.horizontal/2,
				});
			}
		},

		appendItemElems: function(itemElems, point, size) {

			var index = 0,
				_this = this;

			itemElems.imagesLoaded( function() { 

				itemElems.each(function(i, itemElem) {

					var $itemElem = $(itemElem);

					if(index > point.grid.length - 1) {
						index = 0;
					}

					_this.setItemElem($itemElem, point, size, index);

					_this.container.append( $itemElem ).isotope( 'appended', $itemElem ).isotope('layout');

					index ++;

				});

				if ( _this.container.hasClass('wiloke_grid_magnific_popup') ){
					_this.container.wilokeGridPopup();
				}

			});
		},

		getPostId: function() {

			var ids = [];

			this.container.find(this.gridItem).each(function(index, el) {
				var self = $(this);
				ids.push(self.data('post-id'));
			});

			return ids.join(',');
		},

		filterItem: function() {

			if( this.filter ) {

				var _this = this;

				_this.filter.on('click', 'a', function(event) {
					event.preventDefault();

					var self = $(event.currentTarget),
						filter = self.data().filter,
						term = self.data('term-id'),
						total = self.data('total');

					_this.container.isotope({
						filter: filter
					});

					if(typeof term !=='undefined') {

						var length = _this.container.find(filter).length;

						_this.options.data.term = term;

						if( total <= length ) {
							_this.$el.find(_this.button).addClass('wil_btn_more--disable');
						} else {
							_this.$el.find(_this.button).removeClass('wil_btn_more--disable');
						}

					} else {
						_this.options.data.term = 0;
						_this.$el.find(_this.button).removeClass('wil_btn_more--disable');
					}

					_this.filter.find('.active').removeClass('active');

					self.closest('li').addClass('active');

					if(_this.options.data.display === 'lazy') {
						setTimeout( function() { 
	                        Waypoint.refreshAll();
	                    }, 200);
					}

				});

			}
		},

		buttonClick: function() {

			var _this = this,
				paged = 1,
				$button = _this.$el.find(_this.button);

			$button.on('click', '.wil_btn_more__inner', function(event) {
				event.preventDefault();

				var self = $(event.currentTarget).closest(_this.button),
					point = _this.breakpoint(),
					data = _this.getParam(point);
				if( !_this.$el.hasClass('ajax-loading') && !self.hasClass('wil_btn_more--disable') ) {

					$.ajax({
						url: WilokeIsotopeAjax.ajaxurl,
						type: 'POST',
						dataType: 'html',
						data: {
							action: 'wiloke_isotope_ajax',
							data: data,
							security: _this.options.data.nonce
						},
						beforeSend: function() {
							self.addClass('ajax-loading');
							_this.$el.addClass('ajax-loading');
							// console.log('asfsd');
						}
					})
					.done(function(response) {
						response = JSON.parse(response);

						if (response['status'] === 200 ) {
							var size = _this.getSize(point),
								$response = $(response['data']),
								$itemElems = $response.filter(_this.gridItem);

							if($itemElems.length) {
								_this.appendItemElems($itemElems, point, size);
							}
							if ( typeof response.paged !== 'undefined' ){
								paged = parseInt(response.paged, 10);
								_this.options.data.paged = paged+1; 
							}

							switch ( data.type ){
								case 'youtube':
									_this.options.data.youtubeNextpageToken = response.nextPageToken;
									break;
								case 'instagram':
									_this.options.data.nextInstagramPageUrl = response.nextPageUrl;
									break;
							}

							if ( response.loadedAll ){
								self.remove();
							}
						}

						if( response['max_num_pages'] <= 1 ) {

							var $currentFilter = _this.filter.find('.active a'),
								filter = $currentFilter.data('filter');

							if(typeof filter ==='undefined' || filter === '*') {
								_this.$el.find(_this.button).remove();
							} else {
								_this.$el.find(_this.button).addClass('wil_btn_more--disable')
							}

						} else {
							_this.$el.find(_this.button).removeClass('wil_btn_more--disable')
						}
						
						self.removeClass('ajax-loading');

						_this.$el.removeClass('ajax-loading');

						setTimeout( function() { 
	                        _this.$el.removeClass('wil-masonry-loading').find('.wil-masonry-loading-icon').remove();
	                    }, 50);

						if(_this.options.data.display === 'lazy') {
							setTimeout( function() { 
								Waypoint.refreshAll();
		                    }, 200);
						}

	                    // console.log(response['message']);

					});
				}			

			});

			$button.find('.wil_btn_more__inner').trigger('click');
		},

		lazyLoad: function() {

			var _this = this;

			if(_this.options.data.display === 'lazy') {

				_this.$el.find(_this.button).waypoint(function(direction) {
					
	                if( direction === 'down' ) {
                        _this.$el.find(_this.button).find('.wil_btn_more__inner').trigger('click');
	                }
	             
	            }, {
	                offset: '80%'
	            });
            }
		},

		getParam: function(point) {

			var data = this.options.data,
				post__not_in = this.getPostId();

			data.post__not_in = post__not_in;
			data['posts_per_page'] = point.items;

			return data;
		},

		breakpoint: function() {

			var match = -1, point,
				width = window.innerWidth,
				overwrites = this.options.devices;

			if( overwrites ) {

				$.each(overwrites, function(breakpoint) {
					if (breakpoint <= width && breakpoint > match) {
						match = Number(breakpoint);
					}
				});

				if( match != -1 ) {
					point = overwrites[match];
					point['breakpoint'] = match;
				} else {
					var keys = Object.keys(overwrites),
						small =	Math.min(...keys);

					point = overwrites[small];
					point['breakpoint'] = small;
				}

			} else {

				point = {
					breakpoint: 0,
					vertical: 0,
					horizontal: 0,
					column: 1,
					items: 5,
					grid: [{width: 1, height: 1}]
				}
			}

			return point;
		},

		onResize: function() {

			var _this = this,
				setimeout,
				point = _this.breakpoint(),
				breakpoint = point.breakpoint,
				containerSize = _this.getContainerSize();

			$(window).on('resize', function(event) {

				if(setimeout) {
					clearTimeout(setimeout);

				}

				setimeout = setTimeout(function() {
					
					var _point = _this.breakpoint(),
						_breakpoint = _point.breakpoint,
						_containerSize = _this.getContainerSize();
					
					if( _breakpoint != breakpoint ) {
						point = _point;
						breakpoint = _breakpoint;
						containerSize = _containerSize;
						_this.settings(_point);
					} else {
						containerSize = _containerSize;
						_this.refresh();
					}

					clearTimeout(setimeout)

				}, 50);
			});
		}
	}

	window.WilokeIsotope = WilokeIsotope;

	$.fn.WilokeIsotope = function(options) {

        return this.each(function() {

        	var $this = $(this),
        		data = $this.data('WilokeIsotope');

    		if(!data) {
            	data = new WilokeIsotope($this, options);
            	$this.data('WilokeIsotope', data);
            }

        });
    };

}));
